@@ -1,5 +1,5 @@
 /*
- *  Copyright (c) 2012-2023, Peter Haag
+ *  Copyright (c) 2012-2024, Peter Haag
  *  All rights reserved.
  *
  *  Redistribution and use in source and binary forms, with or without
@@ -140,14 +140,18 @@ static const struct ipfixTranslationMap_s {
     {IPFIX_forwardingStatus, SIZEfwdStatus, NumberCopy, EXgenericFlowID, OFFfwdStatus, STACK_NONE, "forwarding status"},
     {IPFIX_tcpControlBits, SIZEtcpFlags, NumberCopy, EXgenericFlowID, OFFtcpFlags, STACK_NONE, "TCP flags"},
     {IPFIX_SourceTransportPort, SIZEsrcPort, NumberCopy, EXgenericFlowID, OFFsrcPort, STACK_NONE, "src port"},
+    {IPFIX_udpSourcePort, SIZEsrcPort, NumberCopy, EXgenericFlowID, OFFsrcPort, STACK_NONE, "src port"},
+    {IPFIX_tcpSourcePort, SIZEsrcPort, NumberCopy, EXgenericFlowID, OFFsrcPort, STACK_NONE, "src port"},
     {IPFIX_SourceIPv4Address, SIZEsrc4Addr, NumberCopy, EXipv4FlowID, OFFsrc4Addr, STACK_NONE, "src IPv4"},
     {IPFIX_SourceIPv4PrefixLength, SIZEsrcMask, NumberCopy, EXflowMiscID, OFFsrcMask, STACK_NONE, "src mask IPv4"},
     {IPFIX_ingressInterface, SIZEinput, NumberCopy, EXflowMiscID, OFFinput, STACK_NONE, "input interface"},
     {IPFIX_DestinationTransportPort, SIZEdstPort, NumberCopy, EXgenericFlowID, OFFdstPort, STACK_DSTPORT, "dst port"},
+    {IPFIX_udpDestinationPort, SIZEdstPort, NumberCopy, EXgenericFlowID, OFFdstPort, STACK_DSTPORT, "dst port"},
+    {IPFIX_tcpDestinationPort, SIZEdstPort, NumberCopy, EXgenericFlowID, OFFdstPort, STACK_DSTPORT, "dst port"},
     {IPFIX_DestinationIPv4Address, SIZEdst4Addr, NumberCopy, EXipv4FlowID, OFFdst4Addr, STACK_NONE, "dst IPv4"},
     {IPFIX_DestinationIPv4PrefixLength, SIZEdstMask, NumberCopy, EXflowMiscID, OFFdstMask, STACK_NONE, "dst mask IPv4"},
     {IPFIX_egressInterface, SIZEoutput, NumberCopy, EXflowMiscID, OFFoutput, STACK_NONE, "output interface"},
-    {IPFIX_ipNextHopIPv4Address, SIZENext4HopIP, NumberCopy, EXipNextHopV4ID, OFFNext4HopIP, STACK_NONE, "IPv4 next hop"},
+    {IPFIX_ipNextHopIPv4Address, SIZENextHopV4IP, NumberCopy, EXipNextHopV4ID, OFFNextHopV4IP, STACK_NONE, "IPv4 next hop"},
     {IPFIX_bgpSourceAsNumber, SIZEsrcAS, NumberCopy, EXasRoutingID, OFFsrcAS, STACK_NONE, "src AS"},
     {IPFIX_bgpDestinationAsNumber, SIZEdstAS, NumberCopy, EXasRoutingID, OFFdstAS, STACK_NONE, "dst AS"},
     {IPFIX_bgpNextHopIPv4Address, SIZEbgp4NextIP, NumberCopy, EXbgpNextHopV4ID, OFFbgp4NextIP, STACK_NONE, "IPv4 bgp next hop"},
@@ -171,13 +175,21 @@ static const struct ipfixTranslationMap_s {
     {IPFIX_postIpClassOfService, SIZEdstTos, NumberCopy, EXflowMiscID, OFFdstTos, STACK_NONE, "post IP class of Service"},
     {IPFIX_SourceMacAddress, SIZEinSrcMac, NumberCopy, EXmacAddrID, OFFinSrcMac, STACK_NONE, "in src MAC addr"},
     {IPFIX_postDestinationMacAddress, SIZEoutDstMac, NumberCopy, EXmacAddrID, OFFoutDstMac, STACK_NONE, "out dst MAC addr"},
-    {IPFIX_vlanId, SIZEsrcVlan, NumberCopy, EXvLanID, OFFsrcVlan, STACK_NONE, "src VLAN ID"},
-    {IPFIX_postVlanId, SIZEdstAS, NumberCopy, EXvLanID, OFFdstAS, STACK_NONE, "dst VLAN ID"},
+    {IPFIX_vlanId, SIZEvlanID, NumberCopy, EXvLanID, OFFvlanID, STACK_NONE, "src VLAN ID"},
+    {IPFIX_postVlanId, SIZEpostVlanID, NumberCopy, EXvLanID, OFFpostVlanID, STACK_NONE, "dst VLAN ID"},
+    {IPFIX_dot1qVlanId, SIZEvlanID, NumberCopy, EXlayer2ID, OFFvlanID, STACK_NONE, "dot1q VLAN ID"},
+    {IPFIX_postDot1qVlanId, SIZEpostVlanID, NumberCopy, EXlayer2ID, OFFpostVlanID, STACK_NONE, "dot1q post VLAN ID"},
+    {IPFIX_dot1qCustomerVlanId, SIZEcustomerVlanId, NumberCopy, EXlayer2ID, OFFcustomerVlanId, STACK_NONE, "dot1q customer VLAN ID"},
+    {IPFIX_postDot1qCustomerVlanId, SIZEpostCustomerVlanId, NumberCopy, EXlayer2ID, OFFpostCustomerVlanId, STACK_NONE, "dot1q post customer VLAN ID"},
+    {IPFIX_ingressPhysicalInterface, SIZEphysIngress, NumberCopy, EXlayer2ID, OFFphysIngress, STACK_NONE, "ingress physical interface ID"},
+    {IPFIX_egressPhysicalInterface, SIZEphysEgress, NumberCopy, EXlayer2ID, OFFphysEgress, STACK_NONE, "egress physical interface ID"},
+    {IPFIX_ipVersion, SIZEipVersion, NumberCopy, EXlayer2ID, OFFipVersion, STACK_NONE, "ip version"},
     {IPFIX_flowDirection, SIZEdir, NumberCopy, EXflowMiscID, OFFdir, STACK_NONE, "flow direction"},
     {IPFIX_biflowDirection, SIZEbiFlowDir, NumberCopy, EXflowMiscID, OFFbiFlowDir, STACK_NONE, "biFlow direction"},
     {IPFIX_flowEndReason, SIZEflowEndReason, NumberCopy, EXflowMiscID, OFFflowEndReason, STACK_NONE, "Flow end reason"},
-    {IPFIX_fragmentFlags, SIZEfragmentFlags, NumberCopy, EXflowMiscID, OFFfragmentFlags, STACK_NONE, "IP fragment flags"},
-    {IPFIX_ipNextHopIPv6Address, SIZENext6HopIP, NumberCopy, EXipNextHopV6ID, OFFNext6HopIP, STACK_NONE, "IPv6 next hop IP"},
+    {IPFIX_ipTTL, SIZEipTTL, NumberCopy, EXipInfoID, OFFipTTL, STACK_NONE, "IP ttl"},
+    {IPFIX_fragmentFlags, SIZEfragmentFlags, NumberCopy, EXipInfoID, OFFfragmentFlags, STACK_NONE, "IP fragment flags"},
+    {IPFIX_ipNextHopIPv6Address, SIZENextHopV6IP, NumberCopy, EXipNextHopV6ID, OFFNextHopV6IP, STACK_NONE, "IPv6 next hop IP"},
     {IPFIX_bgpNextHopIPv6Address, SIZEbgp6NextIP, NumberCopy, EXbgpNextHopV6ID, OFFbgp6NextIP, STACK_NONE, "IPv6 bgp next hop IP"},
     {IPFIX_mplsTopLabelStackSection, SIZEmplsLabel1, NumberCopy, EXmplsLabelID, OFFmplsLabel1, STACK_NONE, "mpls label 1"},
     {IPFIX_mplsLabelStackSection2, SIZEmplsLabel2, NumberCopy, EXmplsLabelID, OFFmplsLabel2, STACK_NONE, "mpls label 2"},
@@ -214,26 +226,32 @@ static const struct ipfixTranslationMap_s {
     {IPFIX_INGRESS_VRFID, SIZEingressVrf, NumberCopy, EXvrfID, OFFingressVrf, STACK_NONE, "ingress VRF ID"},
     {IPFIX_EGRESS_VRFID, SIZEegressVrf, NumberCopy, EXvrfID, OFFegressVrf, STACK_NONE, "egress VRF ID"},
     // NAT
-    {IPFIX_observationTimeMilliseconds, SIZEmsecEvent, NumberCopy, EXnelCommonID, OFFmsecEvent, STACK_MSEC, "msec time event"},
-    {IPFIX_natEvent, SIZEnatEvent, NumberCopy, EXnelCommonID, OFFnatEvent, STACK_NONE, "NAT event"},
-    {IPFIX_postNATSourceIPv4Address, SIZExlateSrc4Addr, NumberCopy, EXnselXlateIPv4ID, OFFxlateSrc4Addr, STACK_NONE, "xlate src addr"},
-    {IPFIX_postNATDestinationIPv4Address, SIZExlateDst4Addr, NumberCopy, EXnselXlateIPv4ID, OFFxlateDst4Addr, STACK_NONE, "xlate dst addr"},
-    {IPFIX_postNAPTSourceTransportPort, SIZExlateSrcPort, NumberCopy, EXnselXlatePortID, OFFxlateSrcPort, STACK_NONE, "xlate src port"},
-    {IPFIX_postNAPTDestinationTransportPort, SIZExlateDstPort, NumberCopy, EXnselXlatePortID, OFFxlateDstPort, STACK_NONE, "xlate dst port"},
+    {IPFIX_observationTimeMilliseconds, SIZEmsecEvent, NumberCopy, EXnatCommonID, OFFmsecEvent, STACK_MSEC, "msec time event"},
+    {IPFIX_natEvent, SIZEnatEvent, NumberCopy, EXnatCommonID, OFFnatEvent, STACK_NONE, "NAT event"},
+    {IPFIX_postNATSourceIPv4Address, SIZExlateSrc4Addr, NumberCopy, EXnatXlateIPv4ID, OFFxlateSrc4Addr, STACK_NONE, "xlate src addr"},
+    {IPFIX_postNATDestinationIPv4Address, SIZExlateDst4Addr, NumberCopy, EXnatXlateIPv4ID, OFFxlateDst4Addr, STACK_NONE, "xlate dst addr"},
+    {IPFIX_postNAPTSourceTransportPort, SIZExlateSrcPort, NumberCopy, EXnatXlatePortID, OFFxlateSrcPort, STACK_NONE, "xlate src port"},
+    {IPFIX_postNAPTDestinationTransportPort, SIZExlateDstPort, NumberCopy, EXnatXlatePortID, OFFxlateDstPort, STACK_NONE, "xlate dst port"},
+    {IPFIX_flowId, SIZEflowId, NumberCopy, EXflowIdID, OFFflowId, STACK_NONE, "flow ID"},
     // cgNAT
-    {IPFIX_NATPOOL_ID, SIZEnatPoolID, NumberCopy, EXnelCommonID, OFFnatPoolID, STACK_NONE, "nat pool ID"},
-    {IPFIX_PORT_BLOCK_START, SIZEnelblockStart, NumberCopy, EXnelXlatePortID, OFFnelblockStart, STACK_NONE, "NAT block start"},
-    {IPFIX_PORT_BLOCK_END, SIZEnelblockEnd, NumberCopy, EXnelXlatePortID, OFFnelblockEnd, STACK_NONE, "NAT block end"},
-    {IPFIX_PORT_BLOCK_STEP, SIZEnelblockStep, NumberCopy, EXnelXlatePortID, OFFnelblockStep, STACK_NONE, "NAT block step"},
-    {IPFIX_PORT_BLOCK_SIZE, SIZEnelblockSize, NumberCopy, EXnelXlatePortID, OFFnelblockSize, STACK_NONE, "NAT block size"},
+    {IPFIX_NATPOOL_ID, SIZEnatPoolID, NumberCopy, EXnatCommonID, OFFnatPoolID, STACK_NONE, "nat pool ID"},
+    {IPFIX_PORT_BLOCK_START, SIZEnelblockStart, NumberCopy, EXnatPortBlockID, OFFnelblockStart, STACK_NONE, "NAT block start"},
+    {IPFIX_PORT_BLOCK_END, SIZEnelblockEnd, NumberCopy, EXnatPortBlockID, OFFnelblockEnd, STACK_NONE, "NAT block end"},
+    {IPFIX_PORT_BLOCK_STEP, SIZEnelblockStep, NumberCopy, EXnatPortBlockID, OFFnelblockStep, STACK_NONE, "NAT block step"},
+    {IPFIX_PORT_BLOCK_SIZE, SIZEnelblockSize, NumberCopy, EXnatPortBlockID, OFFnelblockSize, STACK_NONE, "NAT block size"},
     // inline-monitoring inmon
     {IPFIX_dataLinkFrameSize, SIZEframeSize, NumberCopy, EXinmonMetaID, OFFframeSize, STACK_NONE, "inmon frame size"},
     {IPFIX_dataLinkFrameType, SIZElinkType, NumberCopy, EXinmonMetaID, OFFlinkType, STACK_NONE, "inmon link type"},
     {IPFIX_dataLinkFrameSection, SIZEpacket, ByteCopy, EXinmonFrameID, OFFpacket, STACK_NONE, "inmon packet content"},
 
     // payload
-    {LOCAL_inPayload, VARLENGTH, NumberCopy, EXinPayloadID, 0, STACK_NONE, "in payload"},
-    {LOCAL_outPayload, VARLENGTH, NumberCopy, EXoutPayloadID, 0, STACK_NONE, "out payload"},
+    {LOCAL_inPayload, VARLENGTH, ByteCopy, EXinPayloadID, 0, STACK_NONE, "in payload"},
+    {LOCAL_outPayload, VARLENGTH, ByteCopy, EXoutPayloadID, 0, STACK_NONE, "out payload"},
+
+    // Nokia
+    {NOKIA_InsideServiceId, SIZEinServiceID, NumberCopy, EXnokiaNatID, OFFinServiceID, STACK_NONE, "Nokia inside service ID"},
+    {NOKIA_OutsideServiceId, SIZEoutServiceID, NumberCopy, EXnokiaNatID, OFFoutServiceID, STACK_NONE, "Nokia outside service ID"},
+    {NOKIA_NatSubString, SIZEnatSubString, ByteCopy, EXnokiaNatStringID, OFFnatSubString, STACK_NONE, "Nokia nat substring"},
 
     // End of table
     {0, 0, 0, 0, 0, STACK_NONE, NULL},
@@ -325,10 +343,10 @@ int Init_IPFIX(int verbose, int32_t sampling, char *extensionList) {
 
     if (sampling < 0) {
         LogInfo("Init IPFIX: Max number of ipfix tags enabled: %u, overwrite sampling: %d", tagsEnabled, -defaultSampling);
-        dbg_printf("Initv9: Overwrite sampling: %d\n", -defaultSampling);
+        dbg_printf("Init ipfix: Overwrite sampling: %d\n", -defaultSampling);
     } else {
         LogInfo("Init IPFIX: Max number of ipfix tags enabled: %u, default sampling: %d", tagsEnabled, defaultSampling);
-        dbg_printf("Initv9: Default sampling: %d\n", defaultSampling);
+        dbg_printf("Init ipfix: Default sampling: %d\n", defaultSampling);
     }
 
     return 1;
@@ -363,6 +381,22 @@ static int LookupElement(uint16_t type, uint32_t EnterpriseNumber) {
                     break;
             }
             break;
+        case 637:  // Nokia
+            switch (type) {
+                case 91:  // InsideServiceId
+                    dbg_printf(" NOKIA enterprise InsideServiceId: %u\n", type);
+                    type = NOKIA_InsideServiceId;
+                    break;
+                case 92:  // OutsideServiceId
+                    dbg_printf(" NOKIA enterprise OutsideServiceId: %u\n", type);
+                    type = NOKIA_OutsideServiceId;
+                    break;
+                case 93:  // NatSubString
+                    dbg_printf(" NOKIA enterprise NatSubString: %u\n", type);
+                    type = NOKIA_NatSubString;
+                    break;
+            }
+            break;
         case 6871:  // yaf CERT Coordination Centre
             // map yaf types here
             switch (type) {
@@ -425,7 +459,7 @@ static exporterDomain_t *getExporter(FlowSource_t *fs, uint32_t ObservationDomai
     // nothing found
     *e = (exporterDomain_t *)calloc(1, sizeof(exporterDomain_t));
     if (!(*e)) {
-        LogError("Process_ipfix: Panic! malloc() %s line %d: %s", __FILE__, __LINE__, strerror(errno));
+        LogError("Process_ipfix: Panic! calloc() %s line %d: %s", __FILE__, __LINE__, strerror(errno));
         return NULL;
     }
     (*e)->info.header.type = ExporterInfoRecordType;
@@ -491,7 +525,7 @@ static void InsertSampler(FlowSource_t *fs, exporterDomain_t *exporter, sampler_
         sampler->next = NULL;
         exporter->sampler = sampler;
 
-        AppendToBuffer(fs->nffile, &(sampler->record), sampler->record.size);
+        fs->dataBlock = AppendToBuffer(fs->nffile, fs->dataBlock, &(sampler->record), sampler->record.size);
         LogInfo("Add new sampler id: %lli, algorithm: %u, packet interval: %u, packet space: %u", sampler_record->id, sampler_record->algorithm,
                 sampler_record->packetInterval, sampler_record->spaceInterval);
         dbg_printf("Add new sampler id: %lli, algorithm: %u, packet interval: %u, packet space: %u\n", sampler_record->id, sampler_record->algorithm,
@@ -505,7 +539,7 @@ static void InsertSampler(FlowSource_t *fs, exporterDomain_t *exporter, sampler_
                 // found same sampler id - update record if changed
                 if (sampler_record->algorithm != sampler->record.algorithm || sampler_record->packetInterval != sampler->record.packetInterval ||
                     sampler_record->spaceInterval != sampler->record.spaceInterval) {
-                    AppendToBuffer(fs->nffile, &(sampler->record), sampler->record.size);
+                    fs->dataBlock = AppendToBuffer(fs->nffile, fs->dataBlock, &(sampler->record), sampler->record.size);
                     sampler->record.algorithm = sampler_record->algorithm;
                     sampler->record.packetInterval = sampler_record->packetInterval;
                     sampler->record.spaceInterval = sampler_record->spaceInterval;
@@ -525,7 +559,7 @@ static void InsertSampler(FlowSource_t *fs, exporterDomain_t *exporter, sampler_
                 // end of sampler chain - insert new sampler
                 sampler->next = (sampler_t *)malloc(sizeof(sampler_t));
                 if (!sampler->next) {
-                    LogError("Process_v9: Panic! malloc(): %s line %d: %s", __FILE__, __LINE__, strerror(errno));
+                    LogError("Process_ipfix: Panic! malloc(): %s line %d: %s", __FILE__, __LINE__, strerror(errno));
                     return;
                 }
                 sampler = sampler->next;
@@ -535,7 +569,7 @@ static void InsertSampler(FlowSource_t *fs, exporterDomain_t *exporter, sampler_
                 sampler->record.exporter_sysid = exporter->info.sysid;
                 sampler->next = NULL;
 
-                AppendToBuffer(fs->nffile, &(sampler->record), sampler->record.size);
+                fs->dataBlock = AppendToBuffer(fs->nffile, fs->dataBlock, &(sampler->record), sampler->record.size);
                 LogInfo("Append new sampler id: %lli, algorithm: %u, packet interval: %u, packet space: %u", sampler_record->id,
                         sampler_record->algorithm, sampler_record->packetInterval, sampler_record->spaceInterval);
                 dbg_printf("Append new sampler id: %lli, algorithm: %u, packet interval: %u, packet space: %u\n", sampler_record->id,
@@ -587,9 +621,7 @@ static templateList_t *getTemplate(exporterDomain_t *exporter, uint16_t id) {
 }  // End of getTemplate
 
 static templateList_t *newTemplate(exporterDomain_t *exporter, uint16_t id) {
-    templateList_t *template;
-
-    template = calloc(1, sizeof(templateList_t));
+    templateList_t *template = (templateList_t *)calloc(1, sizeof(templateList_t));
     if (!template) {
         LogError("Process_ipfix: Panic! calloc() %s line %d: %s", __FILE__, __LINE__, strerror(errno));
         return NULL;
@@ -805,7 +837,7 @@ static void Process_ipfix_template_add(exporterDomain_t *exporter, void *DataPtr
             return;
         }
 
-        sequence_t *sequenceTable = malloc((count + 4) * sizeof(sequence_t));  // + 2 for IP and time received
+        sequence_t *sequenceTable = (sequence_t *)malloc((count + 4) * sizeof(sequence_t));  // + 2 for IP and time received
         if (!sequenceTable) {
             LogError("Process_ipfix: malloc(): %s line %d: %s", __FILE__, __LINE__, strerror(errno));
             return;
@@ -866,7 +898,7 @@ static void Process_ipfix_template_add(exporterDomain_t *exporter, void *DataPtr
             LogError("Process_ipfix: abort template add: %s line %d", __FILE__, __LINE__);
             return;
         }
-        dataTemplate_t *dataTemplate = calloc(1, sizeof(dataTemplate_t));
+        dataTemplate_t *dataTemplate = (dataTemplate_t *)calloc(1, sizeof(dataTemplate_t));
         if (!dataTemplate) {
             LogError("Error calloc(): %s in %s:%d", strerror(errno), __FILE__, __LINE__);
             return;
@@ -1231,8 +1263,6 @@ static void Process_ipfix_data(exporterDomain_t *exporter, uint32_t ExportTime,
         receivedSize = ExtensionsEnabled[EXipReceivedV4ID] ? EXipReceivedV4Size : 0;
 
     while (size_left > 0) {
-        void *outBuff;
-
         if (size_left < 4) {  // rounding pads
             size_left = 0;
             continue;
@@ -1241,7 +1271,12 @@ static void Process_ipfix_data(exporterDomain_t *exporter, uint32_t ExportTime,
         // check for enough space in output buffer
         uint32_t outRecordSize = CalcOutRecordSize(sequencer, inBuff, size_left);
 
-        int buffAvail = CheckBufferSpace(fs->nffile, sizeof(recordHeaderV3_t) + outRecordSize + receivedSize);
+        if (!IsAvailable(fs->dataBlock, sizeof(recordHeaderV3_t) + outRecordSize + receivedSize)) {
+            // flush block - get an empty one
+            fs->dataBlock = WriteBlock(fs->nffile, fs->dataBlock);
+        }
+
+        int buffAvail = BlockAvailable(fs->dataBlock);
         if (buffAvail == 0) {
             // this should really never occur, because the buffer gets flushed earlier
             LogError("Process_ipfix: output buffer size error. Skip ipfix record processing");
@@ -1249,9 +1284,10 @@ static void Process_ipfix_data(exporterDomain_t *exporter, uint32_t ExportTime,
             return;
         }
 
+        void *outBuff;
     REDO:
         // map file record to output buffer
-        outBuff = fs->nffile->buff_ptr;
+        outBuff = GetCurrentCursor(fs->dataBlock);
 
         dbg_printf("[%u] Process data record: %u addr: %llu, size_left: %u buff_avail: %u\n", exporter->info.id, processed_records,
                    (long long unsigned)((ptrdiff_t)inBuff - (ptrdiff_t)data_flowset), size_left, buffAvail);
@@ -1282,7 +1318,13 @@ static void Process_ipfix_data(exporterDomain_t *exporter, uint32_t ExportTime,
 
                 // request new and empty buffer
                 LogInfo("Process ipfix: Sequencer run - resize output buffer");
-                buffAvail = CheckBufferSpace(fs->nffile, buffAvail + 1);
+                // request new and empty buffer
+                fs->dataBlock = WriteBlock(fs->nffile, fs->dataBlock);
+                if (fs->dataBlock == NULL) {
+                    return;
+                }
+
+                int buffAvail = BlockAvailable(fs->dataBlock);
                 if (buffAvail == 0) {
                     // this should really never occur, because the buffer gets flushed earlier
                     LogError("Process_ipfix: output buffer size error. Skip ipfix record processing");
@@ -1491,21 +1533,19 @@ static void Process_ipfix_data(exporterDomain_t *exporter, uint32_t ExportTime,
             flow_record_short(stdout, recordHeaderV3);
         }
 
-        fs->nffile->block_header->size += recordHeaderV3->size;
-        fs->nffile->block_header->NumRecords++;
-        fs->nffile->buff_ptr = outBuff;
+        fs->dataBlock->size += recordHeaderV3->size;
+        fs->dataBlock->NumRecords++;
 
         // buffer size sanity check
-        if (fs->nffile->block_header->size > WRITE_BUFFSIZE) {
+        if (fs->dataBlock->size > WRITE_BUFFSIZE) {
             // should never happen
             LogError("### Software error ###: %s line %d", __FILE__, __LINE__);
             LogError("Process ipfix: Output buffer overflow! Flush buffer and skip records.");
-            LogError("Buffer size: %u > %u", fs->nffile->block_header->size, WRITE_BUFFSIZE);
+            LogError("Buffer size: %u > %u", fs->dataBlock->size, WRITE_BUFFSIZE);
 
             // reset buffer
-            fs->nffile->block_header->size = 0;
-            fs->nffile->block_header->NumRecords = 0;
-            fs->nffile->buff_ptr = (void *)((void *)fs->nffile->block_header + sizeof(dataBlock_t));
+            fs->dataBlock->size = 0;
+            fs->dataBlock->NumRecords = 0;
             return;
         }
     }
@@ -1619,13 +1659,12 @@ static void Process_ipfix_nbar_option_data(exporterDomain_t *exporter, FlowSourc
     dbg_printf("nbar elementSize: %zu, totalSize: %zu\n", elementSize, total_size);
 
     // output buffer size check for all expected records
-    if (!CheckBufferSpace(fs->nffile, total_size)) {
-        // fishy! - should never happen. maybe disk full?
-        LogError("Process_nbar_option: output buffer size error. Abort nbar record processing");
-        return;
+    if (!IsAvailable(fs->dataBlock, total_size)) {
+        // flush block - get an empty one
+        fs->dataBlock = WriteBlock(fs->nffile, fs->dataBlock);
     }
 
-    void *outBuff = fs->nffile->buff_ptr;
+    void *outBuff = GetCurrentCursor(fs->dataBlock);
     // push nbar header
     AddArrayHeader(outBuff, nbarHeader, NbarRecordType, elementSize);
 
@@ -1691,9 +1730,8 @@ static void Process_ipfix_nbar_option_data(exporterDomain_t *exporter, FlowSourc
     }
 
     // update data block header
-    fs->nffile->block_header->size += nbarHeader->size;
-    fs->nffile->block_header->NumRecords++;
-    fs->nffile->buff_ptr += nbarHeader->size;
+    fs->dataBlock->size += nbarHeader->size;
+    fs->dataBlock->NumRecords++;
 
     if (size_left > 7) {
         LogInfo("Process nbar data record - %u extra bytes", size_left);
@@ -1754,13 +1792,12 @@ static void Process_ifvrf_option_data(exporterDomain_t *exporter, FlowSource_t *
     dbg_printf("name elementSize: %zu, totalSize: %zu\n", elementSize, total_size);
 
     // output buffer size check for all expected records
-    if (!CheckBufferSpace(fs->nffile, total_size)) {
-        // fishy! - should never happen. maybe disk full?
-        LogError("Process_ifvrf_option: output buffer size error. Abort nbar record processing");
-        return;
+    if (!IsAvailable(fs->dataBlock, total_size)) {
+        // flush block - get an empty one
+        fs->dataBlock = WriteBlock(fs->nffile, fs->dataBlock);
     }
 
-    void *outBuff = fs->nffile->buff_ptr;
+    void *outBuff = GetCurrentCursor(fs->dataBlock);
     // push nbar header
     AddArrayHeader(outBuff, nameHeader, recordType, elementSize);
 
@@ -1815,9 +1852,8 @@ static void Process_ifvrf_option_data(exporterDomain_t *exporter, FlowSource_t *
     }
 
     // update data block header
-    fs->nffile->block_header->size += nameHeader->size;
-    fs->nffile->block_header->NumRecords++;
-    fs->nffile->buff_ptr += nameHeader->size;
+    fs->dataBlock->size += nameHeader->size;
+    fs->dataBlock->NumRecords++;
 
     if (size_left > 7) {
         LogInfo("Process ifvrf data record - %u extra bytes", size_left);
@@ -1827,7 +1863,7 @@ static void Process_ifvrf_option_data(exporterDomain_t *exporter, FlowSource_t *
     dbg_printf("if/vrf name processed: %u records - header: size: %u, type: %u, numelements: %u, elementSize: %u\n", numRecords, nameHeader->size,
                nameHeader->type, nameHeader->numElements, nameHeader->elementSize);
 
-}  // End of Process_v9_ifvrf_option_data
+}  // End of Process_ifvrf_option_data
 
 static void Process_ipfix_SysUpTime_option_data(exporterDomain_t *exporter, templateList_t *template, void *data_flowset) {
     uint32_t size_left = GET_FLOWSET_LENGTH(data_flowset) - 4;  // -4 for data flowset header -> id and length
