@@ -248,7 +248,7 @@ typedef struct ipfix_template_elements_e_s {
 #define IPFIX_postDestinationMacAddress 57
 #define IPFIX_vlanId 58
 #define IPFIX_postVlanId 59
-
+#define IPFIX_ipVersion 60
 #define IPFIX_flowDirection 61
 #define IPFIX_ipNextHopIPv6Address 62
 #define IPFIX_bgpNextHopIPv6Address 63
@@ -277,6 +277,7 @@ typedef struct ipfix_template_elements_e_s {
 #define IPFIX_flowEndReason 136
 #define IPFIX_observationPointId 138
 #define IPFIX_icmpTypeCodeIPv6 139
+#define IPFIX_flowId 148
 #define IPFIX_observationDomainId 149
 #define IPFIX_flowStartSeconds 150
 #define IPFIX_flowEndSeconds 151
@@ -293,6 +294,12 @@ typedef struct ipfix_template_elements_e_s {
 #define IPFIX_icmpTypeV6 178
 #define IPFIX_icmpCodeV6 179
 
+#define IPFIX_udpSourcePort 180
+#define IPFIX_udpDestinationPort 181
+#define IPFIX_tcpSourcePort 182
+#define IPFIX_tcpDestinationPort 183
+
+#define IPFIX_ipTTL 192
 #define IPFIX_fragmentFlags 197
 
 #define IPFIX_postNATSourceIPv4Address 225
@@ -306,6 +313,15 @@ typedef struct ipfix_template_elements_e_s {
 
 #define IPFIX_biflowDirection 239
 
+#define IPFIX_dot1qVlanId 243
+#define IPFIX_postDot1qVlanId 254
+
+#define IPFIX_dot1qCustomerVlanId 245
+#define IPFIX_postDot1qCustomerVlanId 255
+
+#define IPFIX_ingressPhysicalInterface 252
+#define IPFIX_egressPhysicalInterface 253
+
 // sub template IDs
 #define IPFIX_newconnections 278
 #define IPFIX_subTemplateList 292
@@ -354,6 +370,10 @@ typedef struct ipfix_template_elements_e_s {
 #define LOCAL_inPayload 32767
 #define LOCAL_outPayload 32768
 
+#define NOKIA_InsideServiceId 32769
+#define NOKIA_OutsideServiceId 32770
+#define NOKIA_NatSubString 32771
+
 #define VENDOR_BIT_REVERSE 0x4000
 
 /* prototypes */
