@@ -1,5 +1,5 @@
 /*
- *  Copyright (c) 2009-2023, Peter Haag
+ *  Copyright (c) 2009-2024, Peter Haag
  *  Copyright (c) 2004-2008, SWITCH - Teleinformatikdienste fuer Lehre und Forschung
  *  All rights reserved.
  *
@@ -29,13 +29,12 @@
  *
  */
 
-#ifdef HAVE_CONFIG_H
-#include "config.h"
-#endif
+#include "netflow_v1.h"
 
 #include <arpa/inet.h>
 #include <errno.h>
 #include <netinet/in.h>
+#include <stdint.h>
 #include <stdio.h>
 #include <stdlib.h>
 #include <string.h>
@@ -43,13 +42,8 @@
 #include <sys/types.h>
 #include <unistd.h>
 
-#ifdef HAVE_STDINT_H
-#include <stdint.h>
-#endif
-
 #include "collector.h"
 #include "metric.h"
-#include "netflow_v1.h"
 #include "nfdump.h"
 #include "nffile.h"
 #include "nfnet.h"
@@ -117,8 +111,6 @@ static inline exporter_v1_t *getExporter(FlowSource_t *fs, netflow_v1_header_t *
 
 /* functions */
 
-#include "nffile_inline.c"
-
 int Init_v1(int verbose) {
     printRecord = verbose > 2;
     baseRecordSize = sizeof(recordHeaderV3_t) + EXgenericFlowSize + EXipv4FlowSize + EXflowMiscSize + EXipNextHopV4Size;
@@ -143,19 +135,12 @@ static inline exporter_v1_t *getExporter(FlowSource_t *fs, netflow_v1_header_t *
         LogError("Process_v1: malloc() error in %s line %d: %s\n", __FILE__, __LINE__, strerror(errno));
         return NULL;
     }
-    memset((void *)(*e), 0, sizeof(exporter_v1_t));
-    (*e)->next = NULL;
-    (*e)->info.header.type = ExporterInfoRecordType;
-    (*e)->info.header.size = sizeof(exporter_info_record_t);
-    (*e)->info.version = version;
-    (*e)->info.id = 0;
-    (*e)->info.ip = fs->ip;
-    (*e)->info.sa_family = fs->sa_family;
-    (*e)->info.sysid = 0;
-    (*e)->packets = 0;
-    (*e)->flows = 0;
-    (*e)->sequence_failure = 0;
-
+    (**e) = (exporter_v1_t){.info = {.header = {.type = ExporterInfoRecordType, .size = sizeof(exporter_info_record_t)},
+                                     .version = version,
+                                     .id = 0,
+                                     .ip = fs->ip,
+                                     .sa_family = fs->sa_family,
+                                     .sysid = 0}};
     char *ipstr = GetExporterIP(fs);
     if (fs->sa_family == PF_INET6) {
         (*e)->outRecordSize = baseRecordSize + EXipReceivedV6Size;
@@ -174,13 +159,10 @@ static inline exporter_v1_t *getExporter(FlowSource_t *fs, netflow_v1_header_t *
 }  // End of getExporter
 
 void Process_v1(void *in_buff, ssize_t in_buff_cnt, FlowSource_t *fs) {
-    netflow_v1_header_t *v1_header;
-    exporter_v1_t *exporter;
-
     // map v1 data structure to input buffer
-    v1_header = (netflow_v1_header_t *)in_buff;
+    netflow_v1_header_t *v1_header = (netflow_v1_header_t *)in_buff;
 
-    exporter = getExporter(fs, v1_header);
+    exporter_v1_t *exporter = getExporter(fs, v1_header);
     if (!exporter) {
         LogError("Process_v1: NULL Exporter: Abort v1 record processing");
         return;
@@ -193,37 +175,30 @@ void Process_v1(void *in_buff, ssize_t in_buff_cnt, FlowSource_t *fs) {
     // time received for this packet
     uint64_t msecReceived = ((uint64_t)fs->received.tv_sec * 1000LL) + (uint64_t)((uint64_t)fs->received.tv_usec / 1000LL);
 
-    void *outBuff = fs->nffile->buff_ptr;
     int done = 0;
     while (!done) {
-        netflow_v1_record_t *v1_record;
-        /* Process header */
-
         // count check
         uint16_t count = ntohs(v1_header->count);
         if (count > NETFLOW_V1_MAX_RECORDS) {
             LogError("Process_v1: Unexpected record count in header: %i. Abort v1 record processing", count);
-            fs->nffile->buff_ptr = (void *)outBuff;
             return;
         }
 
         // input buffer size check for all expected records
         if (size_left < (NETFLOW_V1_HEADER_LENGTH + count * NETFLOW_V1_RECORD_LENGTH)) {
             LogError("Process_v1: Not enough data to process v1 record. Abort v1 record processing");
-            fs->nffile->buff_ptr = (void *)outBuff;
             return;
         }
 
         // output buffer size check for all expected records
-        if (!CheckBufferSpace(fs->nffile, count * exporter->outRecordSize)) {
-            // fishy! - should never happen. maybe disk full?
-            LogError("Process_v1: output buffer size error. Abort v1 record processing");
-            return;
+        void *outBuff = GetCurrentCursor(fs->dataBlock);
+        if (!IsAvailable(fs->dataBlock, count * exporter->outRecordSize)) {
+            // flush block - get an empty one
+            fs->dataBlock = WriteBlock(fs->nffile, fs->dataBlock);
+            // map output memory buffer
+            outBuff = GetCursor(fs->dataBlock);
         }
 
-        // map output memory buffer
-        outBuff = fs->nffile->buff_ptr;
-
         v1_header->SysUptime = ntohl(v1_header->SysUptime);
         v1_header->unix_secs = ntohl(v1_header->unix_secs);
         v1_header->unix_nsecs = ntohl(v1_header->unix_nsecs);
@@ -233,7 +208,7 @@ void Process_v1(void *in_buff, ssize_t in_buff_cnt, FlowSource_t *fs) {
             ((uint64_t)(v1_header->unix_secs) * 1000 + ((uint64_t)(v1_header->unix_nsecs) / 1000000)) - (uint64_t)(v1_header->SysUptime);
 
         // process all records
-        v1_record = (netflow_v1_record_t *)((pointer_addr_t)v1_header + NETFLOW_V1_HEADER_LENGTH);
+        netflow_v1_record_t *v1_record = (netflow_v1_record_t *)((void *)v1_header + NETFLOW_V1_HEADER_LENGTH);
 
         /* loop over each records associated with this header */
         uint32_t outSize = 0;
@@ -351,7 +326,7 @@ void Process_v1(void *in_buff, ssize_t in_buff_cnt, FlowSource_t *fs) {
             outBuff += recordHeader->size;
             outSize += recordHeader->size;
             // advance input buffer to next flow record
-            v1_record = (netflow_v1_record_t *)((pointer_addr_t)v1_record + NETFLOW_V1_RECORD_LENGTH);
+            v1_record = (netflow_v1_record_t *)((void *)v1_record + NETFLOW_V1_RECORD_LENGTH);
 
             if (recordHeader->size > exporter->outRecordSize) {
                 LogError("Process_v1: Record size check failed! Expected: %u, counted: %u\n", exporter->outRecordSize, recordHeader->size);
@@ -361,9 +336,8 @@ void Process_v1(void *in_buff, ssize_t in_buff_cnt, FlowSource_t *fs) {
         }  // End of foreach v1 record
 
         // update file record size ( -> output buffer size )
-        fs->nffile->block_header->NumRecords += count;
-        fs->nffile->block_header->size += outSize;
-        fs->nffile->buff_ptr = (void *)outBuff;
+        fs->dataBlock->NumRecords += count;
+        fs->dataBlock->size += outSize;
 
         // still to go for this many input bytes
         size_left -= NETFLOW_V1_HEADER_LENGTH + count * NETFLOW_V1_RECORD_LENGTH;
