@@ -1,5 +1,5 @@
 /*
- *  Copyright (c) 2009-2023, Peter Haag
+ *  Copyright (c) 2009-2024, Peter Haag
  *  Copyright (c) 2004-2008, SWITCH - Teleinformatikdienste fuer Lehre und Forschung
  *  All rights reserved.
  *
@@ -153,7 +153,7 @@ static const struct v9TranslationMap_s {
     {NF9_IPV4_DST_ADDR, SIZEdst4Addr, NumberCopy, EXipv4FlowID, OFFdst4Addr, STACK_NONE, "dst IPv4"},
     {NF9_DST_MASK, SIZEdstMask, NumberCopy, EXflowMiscID, OFFdstMask, STACK_NONE, "dst mask IPv4"},
     {NF9_OUTPUT_SNMP, SIZEoutput, NumberCopy, EXflowMiscID, OFFoutput, STACK_NONE, "output interface"},
-    {NF9_V4_NEXT_HOP, SIZENext4HopIP, NumberCopy, EXipNextHopV4ID, OFFNext4HopIP, STACK_NONE, "IPv4 next hop"},
+    {NF9_V4_NEXT_HOP, SIZENextHopV4IP, NumberCopy, EXipNextHopV4ID, OFFNextHopV4IP, STACK_NONE, "IPv4 next hop"},
     {NF9_SRC_AS, SIZEsrcAS, NumberCopy, EXasRoutingID, OFFsrcAS, STACK_NONE, "src AS"},
     {NF9_DST_AS, SIZEdstAS, NumberCopy, EXasRoutingID, OFFdstAS, STACK_NONE, "dst AS"},
     {NF9_BGP_V4_NEXT_HOP, SIZEbgp4NextIP, NumberCopy, EXbgpNextHopV4ID, OFFbgp4NextIP, STACK_NONE, "IPv4 bgp next hop"},
@@ -167,15 +167,22 @@ static const struct v9TranslationMap_s {
     {NF9_IPV6_DST_MASK, SIZEdstMask, NumberCopy, EXflowMiscID, OFFdstMask, STACK_NONE, "dst mask bits"},
     {NF9_ICMP, Stack_ONLY, NumberCopy, EXgenericFlowID, OFFdstPort, STACK_ICMP, "icmp type/code"},
     {NF9_DST_TOS, SIZEdstTos, NumberCopy, EXflowMiscID, OFFdstTos, STACK_NONE, "post IP class of Service"},
-    {NF_F_fragmentFlags, SIZEfragmentFlags, NumberCopy, EXflowMiscID, OFFfragmentFlags, STACK_NONE, "IP fragment flags"},
+    {NF_F_flowEndReason, SIZEflowEndReason, NumberCopy, EXflowMiscID, OFFflowEndReason, STACK_NONE, "Flow end reason"},
+    {NF_F_ipTTL, SIZEipTTL, NumberCopy, EXipInfoID, OFFipTTL, STACK_NONE, "IP ttl"},
+    {NF_F_fragmentFlags, SIZEfragmentFlags, NumberCopy, EXipInfoID, OFFfragmentFlags, STACK_NONE, "IP fragment flags"},
     {NF9_IN_SRC_MAC, SIZEinSrcMac, NumberCopy, EXmacAddrID, OFFinSrcMac, STACK_NONE, "in src MAC addr"},
     {NF9_OUT_DST_MAC, SIZEoutDstMac, NumberCopy, EXmacAddrID, OFFoutDstMac, STACK_NONE, "out dst MAC addr"},
-    {NF9_SRC_VLAN, SIZEsrcVlan, NumberCopy, EXvLanID, OFFsrcVlan, STACK_NONE, "src VLAN ID"},
-    {NF9_DST_VLAN, SIZEdstVlan, NumberCopy, EXvLanID, OFFdstVlan, STACK_NONE, "dst VLAN ID"},
-    {NF_F_dot1qVlanId, SIZEsrcVlan, NumberCopy, EXvLanID, OFFsrcVlan, STACK_NONE, "src VLAN ID"},
-    {NF_F_postDot1qVlanId, SIZEdstVlan, NumberCopy, EXvLanID, OFFdstVlan, STACK_NONE, "dst VLAN ID"},
+    {NF9_SRC_VLAN, SIZEvlanID, NumberCopy, EXvLanID, OFFvlanID, STACK_NONE, "src VLAN ID"},
+    {NF9_DST_VLAN, SIZEvlanID, NumberCopy, EXvLanID, OFFvlanID, STACK_NONE, "dst VLAN ID"},
+    {NF_F_dot1qVlanId, SIZEvlanID, NumberCopy, EXlayer2ID, OFFvlanID, STACK_NONE, "dot1q VLAN ID"},
+    {NF_F_postDot1qVlanId, SIZEvlanID, NumberCopy, EXlayer2ID, OFFvlanID, STACK_NONE, "dot1q post VLAN ID"},
+    {NF_F_dot1qCustomerVlanId, SIZEcustomerVlanId, NumberCopy, EXlayer2ID, OFFcustomerVlanId, STACK_NONE, "dot1q customer VLAN ID"},
+    {NF_F_postDot1qCustomerVlanId, SIZEpostCustomerVlanId, NumberCopy, EXlayer2ID, OFFpostCustomerVlanId, STACK_NONE, "dot1q post customer VLAN ID"},
+    {NF_F_ingressPhysicalInterface, SIZEphysIngress, NumberCopy, EXlayer2ID, OFFphysIngress, STACK_NONE, "ingress physical interface ID"},
+    {NF_F_egressPhysicalInterface, SIZEphysEgress, NumberCopy, EXlayer2ID, OFFphysEgress, STACK_NONE, "egress physical interface ID"},
+    {NF_9_IP_PROTOCOL_VERSION, SIZEipVersion, NumberCopy, EXlayer2ID, OFFipVersion, STACK_NONE, "ip version"},
     {NF9_DIRECTION, SIZEdir, NumberCopy, EXflowMiscID, OFFdir, STACK_NONE, "flow direction"},
-    {NF9_V6_NEXT_HOP, SIZENext6HopIP, NumberCopy, EXipNextHopV6ID, OFFNext6HopIP, STACK_NONE, "IPv6 next hop IP"},
+    {NF9_V6_NEXT_HOP, SIZENextHopV6IP, NumberCopy, EXipNextHopV6ID, OFFNextHopV6IP, STACK_NONE, "IPv6 next hop IP"},
     {NF9_BPG_V6_NEXT_HOP, SIZEbgp6NextIP, NumberCopy, EXbgpNextHopV6ID, OFFbgp6NextIP, STACK_NONE, "IPv6 bgp next hop IP"},
     {NF_F_BGP_ADJ_NEXT_AS, SIZEnextAdjacentAS, NumberCopy, EXasAdjacentID, OFFnextAdjacentAS, STACK_NONE, "bgb adj next AS"},
     {NF_F_BGP_ADJ_PREV_AS, SIZEprevAdjacentAS, NumberCopy, EXasAdjacentID, OFFprevAdjacentAS, STACK_NONE, "bgb adj prev AS"},
@@ -199,6 +206,7 @@ static const struct v9TranslationMap_s {
     {LOCAL_IPv4Received, SIZEReceived4IP, NumberCopy, EXipReceivedV4ID, OFFReceived4IP, STACK_NONE, "IPv4 exporter"},
     {LOCAL_IPv6Received, SIZEReceived6IP, NumberCopy, EXipReceivedV6ID, OFFReceived6IP, STACK_NONE, "IPv6 exporter"},
     {LOCAL_msecTimeReceived, SIZEmsecReceived, NumberCopy, EXgenericFlowID, OFFmsecReceived, STACK_NONE, "msec time received"},
+    {NF9_ETHERTYPE, SIZEetherType, NumberCopy, EXlayer2ID, OFFetherType, STACK_NONE, "ethertype"},
 
     // NSEL extensions
     {NF_F_FLOW_BYTES, SIZEinBytes, NumberCopy, EXgenericFlowID, OFFinBytes, STACK_NONE, "ASA inBytes total"},
@@ -212,16 +220,16 @@ static const struct v9TranslationMap_s {
     {NF_F_FW_EVENT, SIZEfwEvent, NumberCopy, EXnselCommonID, OFFfwEvent, STACK_NONE, "fw event ID"},
     {NF_F_FW_EVENT_84, SIZEfwEvent, NumberCopy, EXnselCommonID, OFFfwEvent, STACK_NONE, "fw event ID"},
     {NF_F_FW_EXT_EVENT, SIZEfwXevent, NumberCopy, EXnselCommonID, OFFfwXevent, STACK_NONE, "fw ext event ID"},
-    {NF_F_XLATE_SRC_ADDR_IPV4, SIZExlateSrc4Addr, NumberCopy, EXnselXlateIPv4ID, OFFxlateSrc4Addr, STACK_NONE, "xlate src addr"},
-    {NF_F_XLATE_SRC_ADDR_84, SIZExlateSrc4Addr, NumberCopy, EXnselXlateIPv4ID, OFFxlateSrc4Addr, STACK_NONE, "xlate src addr"},
-    {NF_F_XLATE_DST_ADDR_IPV4, SIZExlateDst4Addr, NumberCopy, EXnselXlateIPv4ID, OFFxlateDst4Addr, STACK_NONE, "xlate dst addr"},
-    {NF_F_XLATE_DST_ADDR_84, SIZExlateDst4Addr, NumberCopy, EXnselXlateIPv4ID, OFFxlateDst4Addr, STACK_NONE, "xlate dst addr"},
-    {NF_F_XLATE_SRC_ADDR_IPV6, SIZExlateSrc6Addr, NumberCopy, EXnselXlateIPv6ID, OFFxlateSrc6Addr, STACK_NONE, "xlate src addr"},
-    {NF_F_XLATE_DST_ADDR_IPV6, SIZExlateDst6Addr, NumberCopy, EXnselXlateIPv6ID, OFFxlateDst6Addr, STACK_NONE, "xlate dst addr"},
-    {NF_F_XLATE_SRC_PORT, SIZExlateSrcPort, NumberCopy, EXnselXlatePortID, OFFxlateSrcPort, STACK_NONE, "xlate src port"},
-    {NF_F_XLATE_DST_PORT, SIZExlateDstPort, NumberCopy, EXnselXlatePortID, OFFxlateDstPort, STACK_NONE, "xlate dst port"},
-    {NF_F_XLATE_SRC_PORT_84, SIZExlateSrcPort, NumberCopy, EXnselXlatePortID, OFFxlateSrcPort, STACK_NONE, "xlate src port"},
-    {NF_F_XLATE_DST_PORT_84, SIZExlateDstPort, NumberCopy, EXnselXlatePortID, OFFxlateDstPort, STACK_NONE, "xlate dst port"},
+    {NF_F_XLATE_SRC_ADDR_IPV4, SIZExlateSrc4Addr, NumberCopy, EXnatXlateIPv4ID, OFFxlateSrc4Addr, STACK_NONE, "xlate src addr"},
+    {NF_F_XLATE_SRC_ADDR_84, SIZExlateSrc4Addr, NumberCopy, EXnatXlateIPv4ID, OFFxlateSrc4Addr, STACK_NONE, "xlate src addr"},
+    {NF_F_XLATE_DST_ADDR_IPV4, SIZExlateDst4Addr, NumberCopy, EXnatXlateIPv4ID, OFFxlateDst4Addr, STACK_NONE, "xlate dst addr"},
+    {NF_F_XLATE_DST_ADDR_84, SIZExlateDst4Addr, NumberCopy, EXnatXlateIPv4ID, OFFxlateDst4Addr, STACK_NONE, "xlate dst addr"},
+    {NF_F_XLATE_SRC_ADDR_IPV6, SIZExlateSrc6Addr, NumberCopy, EXnatXlateIPv6ID, OFFxlateSrc6Addr, STACK_NONE, "xlate src addr"},
+    {NF_F_XLATE_DST_ADDR_IPV6, SIZExlateDst6Addr, NumberCopy, EXnatXlateIPv6ID, OFFxlateDst6Addr, STACK_NONE, "xlate dst addr"},
+    {NF_F_XLATE_SRC_PORT, SIZExlateSrcPort, NumberCopy, EXnatXlatePortID, OFFxlateSrcPort, STACK_NONE, "xlate src port"},
+    {NF_F_XLATE_DST_PORT, SIZExlateDstPort, NumberCopy, EXnatXlatePortID, OFFxlateDstPort, STACK_NONE, "xlate dst port"},
+    {NF_F_XLATE_SRC_PORT_84, SIZExlateSrcPort, NumberCopy, EXnatXlatePortID, OFFxlateSrcPort, STACK_NONE, "xlate src port"},
+    {NF_F_XLATE_DST_PORT_84, SIZExlateDstPort, NumberCopy, EXnatXlatePortID, OFFxlateDstPort, STACK_NONE, "xlate dst port"},
     {NF_F_INGRESS_ACL_ID, SIZEingressAcl, NumberCopy, EXnselAclID, OFFingressAcl, STACK_NONE, "ingress ACL ID"},
     {NF_F_EGRESS_ACL_ID, SIZEegressAcl, NumberCopy, EXnselAclID, OFFegressAcl, STACK_NONE, "egress ACL ID"},
     {NF_F_USERNAME, SIZEusername, NumberCopy, EXnselUserID, OFFusername, STACK_NONE, "AAA username"},
@@ -229,12 +237,12 @@ static const struct v9TranslationMap_s {
     {NF_N_EGRESS_VRFID, SIZEegressVrf, NumberCopy, EXvrfID, OFFegressVrf, STACK_NONE, "egress VRF ID"},
 
     // NEL
-    {NF_N_NAT_EVENT, SIZEnatEvent, NumberCopy, EXnelCommonID, OFFnatEvent, STACK_NONE, "NAT event"},
-    {NF_N_NATPOOL_ID, SIZEnatPoolID, NumberCopy, EXnelCommonID, OFFnatPoolID, STACK_NONE, "nat pool ID"},
-    {NF_F_XLATE_PORT_BLOCK_START, SIZEnelblockStart, NumberCopy, EXnelXlatePortID, OFFnelblockStart, STACK_NONE, "NAT block start"},
-    {NF_F_XLATE_PORT_BLOCK_END, SIZEnelblockEnd, NumberCopy, EXnelXlatePortID, OFFnelblockEnd, STACK_NONE, "NAT block end"},
-    {NF_F_XLATE_PORT_BLOCK_STEP, SIZEnelblockStep, NumberCopy, EXnelXlatePortID, OFFnelblockStep, STACK_NONE, "NAT block step"},
-    {NF_F_XLATE_PORT_BLOCK_SIZE, SIZEnelblockSize, NumberCopy, EXnelXlatePortID, OFFnelblockSize, STACK_NONE, "NAT block size"},
+    {NF_N_NAT_EVENT, SIZEnatEvent, NumberCopy, EXnatCommonID, OFFnatEvent, STACK_NONE, "NAT event"},
+    {NF_N_NATPOOL_ID, SIZEnatPoolID, NumberCopy, EXnatCommonID, OFFnatPoolID, STACK_NONE, "nat pool ID"},
+    {NF_F_XLATE_PORT_BLOCK_START, SIZEnelblockStart, NumberCopy, EXnatPortBlockID, OFFnelblockStart, STACK_NONE, "NAT block start"},
+    {NF_F_XLATE_PORT_BLOCK_END, SIZEnelblockEnd, NumberCopy, EXnatPortBlockID, OFFnelblockEnd, STACK_NONE, "NAT block end"},
+    {NF_F_XLATE_PORT_BLOCK_STEP, SIZEnelblockStep, NumberCopy, EXnatPortBlockID, OFFnelblockStep, STACK_NONE, "NAT block step"},
+    {NF_F_XLATE_PORT_BLOCK_SIZE, SIZEnelblockSize, NumberCopy, EXnatPortBlockID, OFFnelblockSize, STACK_NONE, "NAT block size"},
 
     // Nprobe latency
     {NF_NPROBE_CLIENT_NW_DELAY_USEC, SIZEusecClientNwDelay, NumberCopy, EXlatencyID, OFFusecClientNwDelay, STACK_NONE, "nprobe client latency usec"},
@@ -397,7 +405,7 @@ static inline exporterDomain_t *getExporter(FlowSource_t *fs, uint32_t exporter_
     // nothing found
     *e = (exporterDomain_t *)calloc(1, sizeof(exporterDomain_t));
     if (!(*e)) {
-        LogError("Process_v9: Panic! malloc() %s line %d: %s", __FILE__, __LINE__, strerror(errno));
+        LogError("Process_v9: Panic! calloc() %s line %d: %s", __FILE__, __LINE__, strerror(errno));
         return NULL;
     }
     (*e)->info.header.type = ExporterInfoRecordType;
@@ -462,7 +470,7 @@ static void InsertSampler(FlowSource_t *fs, exporterDomain_t *exporter, sampler_
         sampler->next = NULL;
         exporter->sampler = sampler;
 
-        AppendToBuffer(fs->nffile, &(sampler->record), sampler->record.size);
+        fs->dataBlock = AppendToBuffer(fs->nffile, fs->dataBlock, &(sampler->record), sampler->record.size);
         LogInfo("Add new sampler id: %lli, algorithm: %u, packet interval: %u, packet space: %u", sampler_record->id, sampler_record->algorithm,
                 sampler_record->packetInterval, sampler_record->spaceInterval);
 
@@ -477,7 +485,7 @@ static void InsertSampler(FlowSource_t *fs, exporterDomain_t *exporter, sampler_
                     sampler->record.algorithm = sampler_record->algorithm;
                     sampler->record.packetInterval = sampler_record->packetInterval;
                     sampler->record.spaceInterval = sampler_record->spaceInterval;
-                    AppendToBuffer(fs->nffile, &(sampler->record), sampler->record.size);
+                    fs->dataBlock = AppendToBuffer(fs->nffile, fs->dataBlock, &(sampler->record), sampler->record.size);
                     LogInfo("Update existing sampler id: %lli, algorithm: %u, packet interval: %u, packet space: %u", sampler_record->id,
                             sampler_record->algorithm, sampler_record->packetInterval, sampler_record->spaceInterval);
                 } else {
@@ -502,7 +510,7 @@ static void InsertSampler(FlowSource_t *fs, exporterDomain_t *exporter, sampler_
                 sampler->record.exporter_sysid = exporter->info.sysid;
                 sampler->next = NULL;
 
-                AppendToBuffer(fs->nffile, &(sampler->record), sampler->record.size);
+                fs->dataBlock = AppendToBuffer(fs->nffile, fs->dataBlock, &(sampler->record), sampler->record.size);
                 LogInfo("Append new sampler id: %lli, algorithm: %u, packet interval: %u, packet space: %u", sampler_record->id,
                         sampler_record->algorithm, sampler_record->packetInterval, sampler_record->spaceInterval);
                 break;
@@ -550,9 +558,7 @@ static templateList_t *getTemplate(exporterDomain_t *exporter, uint16_t id) {
 }  // End of getTemplate
 
 static templateList_t *newTemplate(exporterDomain_t *exporter, uint16_t id) {
-    templateList_t *template;
-
-    template = calloc(1, sizeof(templateList_t));
+    templateList_t *template = (templateList_t *)calloc(1, sizeof(templateList_t));
     if (!template) {
         LogError("Process_v9: Panic! calloc() %s line %d: %s", __FILE__, __LINE__, strerror(errno));
         return NULL;
@@ -642,7 +648,7 @@ static inline void Process_v9_templates(exporterDomain_t *exporter, void *DataPt
             return;
         }
 
-        sequence_t *sequenceTable = malloc((count + 4) * sizeof(sequence_t));  // + 2 for IP and time received
+        sequence_t *sequenceTable = (sequence_t *)malloc((count + 4) * sizeof(sequence_t));  // + 2 for IP and time received
         if (!sequenceTable) {
             LogError("Process_v9: malloc(): %s line %d: %s", __FILE__, __LINE__, strerror(errno));
             return;
@@ -713,7 +719,7 @@ static inline void Process_v9_templates(exporterDomain_t *exporter, void *DataPt
             LogError("Process_v9: abort template add: %s line %d", __FILE__, __LINE__);
             return;
         }
-        dataTemplate_t *dataTemplate = calloc(1, sizeof(dataTemplate_t));
+        dataTemplate_t *dataTemplate = (dataTemplate_t *)calloc(1, sizeof(dataTemplate_t));
         if (!dataTemplate) {
             LogError("Error calloc(): %s in %s:%d", strerror(errno), __FILE__, __LINE__);
             return;
@@ -1003,8 +1009,6 @@ static inline void Process_v9_data(exporterDomain_t *exporter, void *data_flowse
         receivedSize = ExtensionsEnabled[EXipReceivedV4ID] ? EXipReceivedV4Size : 0;
 
     while (size_left > 0) {
-        void *outBuff;
-
         if (size_left < 4) {  // rounding pads
             size_left = 0;
             continue;
@@ -1012,17 +1016,23 @@ static inline void Process_v9_data(exporterDomain_t *exporter, void *data_flowse
 
         // check for enough space in output buffer
         uint32_t outRecordSize = CalcOutRecordSize(sequencer, inBuff, size_left);
-        int buffAvail = CheckBufferSpace(fs->nffile, sizeof(recordHeaderV3_t) + outRecordSize + receivedSize);
+        if (!IsAvailable(fs->dataBlock, sizeof(recordHeaderV3_t) + outRecordSize + receivedSize)) {
+            // flush block - get an empty one
+            fs->dataBlock = WriteBlock(fs->nffile, fs->dataBlock);
+        }
+
+        int buffAvail = BlockAvailable(fs->dataBlock);
         if (buffAvail == 0) {
             // this should really never occur, because the buffer gets flushed earlier
-            LogError("Process_v9: output buffer size error. Skip ipfix record processing");
-            dbg_printf("Process_v9: output buffer size error. Skip ipfix record processing");
+            LogError("Process_v9: output buffer size error. Skip v9 record processing");
+            dbg_printf("Process_v9: output buffer size error. Skip v9 record processing");
             return;
         }
 
+        void *outBuff;
     REDO:
         // map file record to output buffer
-        outBuff = fs->nffile->buff_ptr;
+        outBuff = GetCurrentCursor(fs->dataBlock);
 
         dbg_printf("[%u] Process data record: %u addr: %llu, size_left: %u buff_avail: %u\n", exporter->info.id, processed_records,
                    (long long unsigned)((ptrdiff_t)inBuff - (ptrdiff_t)data_flowset), size_left, buffAvail);
@@ -1053,23 +1063,26 @@ static inline void Process_v9_data(exporterDomain_t *exporter, void *data_flowse
                     return;
                 }
 
-                // request new and empty buffer
                 LogVerbose("Process v9: Sequencer run - resize output buffer");
-                buffAvail = CheckBufferSpace(fs->nffile, buffAvail + 1);
+                // request new and empty buffer
+                fs->dataBlock = WriteBlock(fs->nffile, fs->dataBlock);
+                if (fs->dataBlock == NULL) {
+                    return;
+                }
+
+                int buffAvail = BlockAvailable(fs->dataBlock);
                 if (buffAvail == 0) {
-                    // this should really never occur, because the buffer gets flushed earlier
-                    LogError("Process_v9: output buffer size error. Skip ipfix record processing");
-                    dbg_printf("Process_v9: output buffer size error. Skip ipfix record processing");
+                    // this should really never happen, because the buffer got flushed
+                    LogError("Process_v9: output buffer size error. Skip v9 record processing");
+                    dbg_printf("Process_v9: output buffer size error. Skip v9 record processing");
                     return;
                 }
                 goto REDO;
                 break;
         }
 
-        dbg_printf(
-            "New record added with %u elements and size: %u, sequencer inLength: %lu, outLength: "
-            "%lu\n",
-            recordHeaderV3->numElements, recordHeaderV3->size, sequencer->inLength, sequencer->outLength);
+        dbg_printf("New record added with %u elements and size: %u, sequencer inLength: %lu, outLength: %lu\n", recordHeaderV3->numElements,
+                   recordHeaderV3->size, sequencer->inLength, sequencer->outLength);
 
         // add router IP
         if (fs->sa_family == PF_INET6) {
@@ -1256,9 +1269,9 @@ static inline void Process_v9_data(exporterDomain_t *exporter, void *data_flowse
             }
             SetFlag(recordHeaderV3->flags, V3_FLAG_EVENT);
         }
-        EXnelCommon_t *nelCommon = sequencer->offsetCache[EXnelCommonID];
-        if (nelCommon) {
-            nelCommon->msecEvent = stack[STACK_MSEC];
+        EXnatCommon_t *natCommon = sequencer->offsetCache[EXnatCommonID];
+        if (natCommon) {
+            natCommon->msecEvent = stack[STACK_MSEC];
             if (genericFlow) {
                 genericFlow->msecFirst = stack[STACK_MSEC];
                 genericFlow->msecLast = stack[STACK_MSEC];
@@ -1278,21 +1291,19 @@ static inline void Process_v9_data(exporterDomain_t *exporter, void *data_flowse
             flow_record_short(stdout, recordHeaderV3);
         }
 
-        fs->nffile->block_header->size += recordHeaderV3->size;
-        fs->nffile->block_header->NumRecords++;
-        fs->nffile->buff_ptr = outBuff;
+        fs->dataBlock->size += recordHeaderV3->size;
+        fs->dataBlock->NumRecords++;
 
         // buffer size sanity check
-        if (fs->nffile->block_header->size > WRITE_BUFFSIZE) {
+        if (fs->dataBlock->size > WRITE_BUFFSIZE) {
             // should never happen
             LogError("### Software error ###: %s line %d", __FILE__, __LINE__);
             LogError("Process v9: Output buffer overflow! Flush buffer and skip records.");
-            LogError("Buffer size: %u > %u", fs->nffile->block_header->size, WRITE_BUFFSIZE);
+            LogError("Buffer size: %u > %u", fs->dataBlock->size, WRITE_BUFFSIZE);
 
             // reset buffer
-            fs->nffile->block_header->size = 0;
-            fs->nffile->block_header->NumRecords = 0;
-            fs->nffile->buff_ptr = (void *)((pointer_addr_t)fs->nffile->block_header + sizeof(dataBlock_t));
+            fs->dataBlock->size = 0;
+            fs->dataBlock->NumRecords = 0;
             return;
         }
     }
@@ -1409,13 +1420,12 @@ static void Process_v9_nbar_option_data(exporterDomain_t *exporter, FlowSource_t
     dbg_printf("nbar elementSize: %zu, totalSize: %zu\n", elementSize, total_size);
 
     // output buffer size check for all expected records
-    if (!CheckBufferSpace(fs->nffile, total_size)) {
-        // fishy! - should never happen. maybe disk full?
-        LogError("Process_nbar_option: output buffer size error. Abort nbar record processing");
-        return;
+    if (!IsAvailable(fs->dataBlock, total_size)) {
+        // flush block - get an empty one
+        fs->dataBlock = WriteBlock(fs->nffile, fs->dataBlock);
     }
 
-    void *outBuff = fs->nffile->buff_ptr;
+    void *outBuff = GetCurrentCursor(fs->dataBlock);
     // push nbar header
     AddArrayHeader(outBuff, nbarHeader, NbarRecordType, elementSize);
 
@@ -1478,9 +1488,8 @@ static void Process_v9_nbar_option_data(exporterDomain_t *exporter, FlowSource_t
     }
 
     // update data block header
-    fs->nffile->block_header->size += nbarHeader->size;
-    fs->nffile->block_header->NumRecords++;
-    fs->nffile->buff_ptr += nbarHeader->size;
+    fs->dataBlock->size += nbarHeader->size;
+    fs->dataBlock->NumRecords++;
 
     if (size_left > 7) {
         LogVerbose("Process nbar data record - %u extra bytes", size_left);
@@ -1542,13 +1551,13 @@ static void Process_v9_ifvrf_option_data(exporterDomain_t *exporter, FlowSource_
     dbg_printf("name elementSize: %zu, totalSize: %zu\n", elementSize, total_size);
 
     // output buffer size check for all expected records
-    if (!CheckBufferSpace(fs->nffile, total_size)) {
-        // fishy! - should never happen. maybe disk full?
-        LogError("Process_ifvrf_option: output buffer size error. Abort nbar record processing");
-        return;
+    if (!IsAvailable(fs->dataBlock, total_size)) {
+        // flush block - get an empty one
+        fs->dataBlock = WriteBlock(fs->nffile, fs->dataBlock);
     }
 
-    void *outBuff = fs->nffile->buff_ptr;
+    void *outBuff = GetCurrentCursor(fs->dataBlock);
+
     // push nbar header
     AddArrayHeader(outBuff, nameHeader, recordType, elementSize);
 
@@ -1603,9 +1612,8 @@ static void Process_v9_ifvrf_option_data(exporterDomain_t *exporter, FlowSource_
     }
 
     // update data block header
-    fs->nffile->block_header->size += nameHeader->size;
-    fs->nffile->block_header->NumRecords++;
-    fs->nffile->buff_ptr += nameHeader->size;
+    fs->dataBlock->size += nameHeader->size;
+    fs->dataBlock->NumRecords++;
 
     if (size_left > 7) {
         LogVerbose("Process ifvrf data record - %u extra bytes", size_left);
@@ -1629,7 +1637,7 @@ static void Process_v9_SysUpTime_option_data(exporterDomain_t *exporter, templat
         exporter->SysUpTime = Get_val(in, optionTemplate->SysUpOption.offset, optionTemplate->SysUpOption.length);
         dbg_printf("Extracted SysUpTime : %llu\n", exporter->SysUpTime);
     } else {
-        LogError("Process_ipfix_option: %s line %d: Not enough data for option data", __FILE__, __LINE__);
+        LogError("Process_v9_option: %s line %d: Not enough data for option data", __FILE__, __LINE__);
         return;
     }
 
@@ -1800,4 +1808,4 @@ void Process_v9(void *in_buff, ssize_t in_buff_cnt, FlowSource_t *fs) {
 
     return;
 
-} /* End of Process_v9 */
\ No newline at end of file
+} /* End of Process_v9 */
