@@ -1,5 +1,5 @@
 /*
- *  Copyright (c) 2012-2023, Peter Haag
+ *  Copyright (c) 2012-2024, Peter Haag
  *
  *  Redistribution and use in source and binary forms, with or without
  *  modification, are permitted provided that the following conditions are met:
@@ -27,9 +27,7 @@
  *
  */
 
-#ifdef HAVE_CONFIG_H
-#include "config.h"
-#endif
+#include "exporter.h"
 
 #include <arpa/inet.h>
 #include <ctype.h>
@@ -44,27 +42,41 @@
 #include <sys/types.h>
 #include <unistd.h>
 
-#include "exporter.h"
 #include "nfdump.h"
 #include "nfxV3.h"
 #include "util.h"
 
 /* global */
-exporter_t **exporter_list;
+static exporter_t **exporter_list = NULL;
 
 /* local variables */
 #define MAX_EXPORTERS 65536
+
+static const struct versionString_s {
+    uint16_t version;
+    char *string;
+} versionString[] = {{5, "netflow v5"}, {9, "netflow v9"}, {10, "ipfix v10"}, {9999, "sflow"}, {0, NULL}};
+
+/* local prototypes */
 static exporter_t *exporter_root;
 
+static char *getVersionString(uint16_t nfversion);
+
 #include "nffile_inline.c"
 
-/* local prototypes */
+static char *getVersionString(uint16_t nfversion) {
+    for (int i = 0; versionString[i].string != NULL; i++) {
+        if (nfversion == versionString[i].version) return versionString[i].string;
+    }
+    return "Unknown version";
+
+}  // End of getVersionString
 
 /* functions */
 int InitExporterList(void) {
-    exporter_list = calloc(MAX_EXPORTERS, sizeof(exporter_t *));
+    exporter_list = (exporter_t **)calloc(MAX_EXPORTERS, sizeof(exporter_t *));
     if (!exporter_list) {
-        LogError("malloc() error in %s line %d: %s\n", __FILE__, __LINE__, strerror(errno));
+        LogError("calloc() error in %s line %d: %s\n", __FILE__, __LINE__, strerror(errno));
         return 0;
     }
     exporter_root = NULL;
@@ -78,6 +90,8 @@ int AddExporterInfo(exporter_info_record_t *exporter_record) {
         return 0;
     }
 
+    if (exporter_list == NULL) InitExporterList();
+
     // sanity check
     uint32_t id = exporter_record->sysid;
     if (id >= MAX_EXPORTERS) {
@@ -158,14 +172,14 @@ int AddExporterInfo(exporter_info_record_t *exporter_record) {
     return 1;
 }  // End of AddExporterInfo
 
-int AddSamplerInfo(sampler_record_t *sampler_record) {
-    if (sampler_record->size != sizeof(sampler_record_t) && sampler_record->size != sizeof(samplerV0_record_t)) {
-        LogError("Corrupt sampler record in %s line %d\n", __FILE__, __LINE__);
+int AddSamplerLegacyRecord(samplerV0_record_t *sampler_record) {
+    if (sampler_record->size != sizeof(samplerV0_record_t)) {
+        LogError("Corrupt sampler record in %s line %d", __FILE__, __LINE__);
         return 0;
     }
 
     sampler_record_t convert_record = {0};
-    sampler_record_t *record = sampler_record;
+    sampler_record_t *record = &convert_record;
     if (sampler_record->size == sizeof(samplerV0_record_t)) {
         samplerV0_record_t *samplerV0_record = (samplerV0_record_t *)sampler_record;
 
@@ -178,13 +192,19 @@ int AddSamplerInfo(sampler_record_t *sampler_record) {
         convert_record.spaceInterval = samplerV0_record->interval - 1;
         record = &convert_record;
     }
+    return AddSamplerRecord(record);
 
-    uint32_t id = record->exporter_sysid;
+}  // End of AddSamplerLegacyRecord
+
+int AddSamplerRecord(sampler_record_t *sampler_record) {
+    uint32_t id = sampler_record->exporter_sysid;
     if (id >= MAX_EXPORTERS) {
         LogError("Corrupt sampler record in %s line %d\n", __FILE__, __LINE__);
         return 0;
     }
 
+    if (exporter_list == NULL) InitExporterList();
+
     if (!exporter_list[id]) {
         LogError("Exporter SysID: %u not found! - Skip sampler record", id);
         return 0;
@@ -192,10 +212,10 @@ int AddSamplerInfo(sampler_record_t *sampler_record) {
 
     sampler_t **sampler = &exporter_list[id]->sampler;
     while (*sampler) {
-        if (memcmp((void *)&(*sampler)->record, (void *)record, sizeof(sampler_record_t)) == 0) {
+        if (memcmp((void *)&(*sampler)->record, (void *)sampler_record, sizeof(sampler_record_t)) == 0) {
             // Found identical sampler already registered
-            dbg_printf("Identical sampler already registered: %u, algorithm: %u, packet interval: %u, packet space: %u\n", record->exporter_sysid,
-                       record->algorithm, record->packetInterval, record->spaceInterval);
+            dbg_printf("Identical sampler already registered: %u, algorithm: %u, packet interval: %u, packet space: %u\n",
+                       sampler_record->exporter_sysid, sampler_record->algorithm, sampler_record->packetInterval, sampler_record->spaceInterval);
             return 2;
         }
         sampler = &((*sampler)->next);
@@ -207,25 +227,25 @@ int AddSamplerInfo(sampler_record_t *sampler_record) {
         return 0;
     }
     (*sampler)->next = NULL;
-    record->exporter_sysid = exporter_list[id]->info.sysid;
+    sampler_record->exporter_sysid = exporter_list[id]->info.sysid;
 
-    memcpy((void *)&(*sampler)->record, (void *)record, sizeof(sampler_record_t));
+    memcpy((void *)&(*sampler)->record, (void *)sampler_record, sizeof(sampler_record_t));
     dbg_printf("Insert sampler record for exporter at slot %i:\n", id);
 
 #ifdef DEVEL
     {
         if (sampler_record->id < 0) {
-            printf("Exporter SysID: %u,	Generic Sampler: algorithm: %u, packet interval: %u, packet space: %u\n", record->exporter_sysid,
-                   record->algorithm, record->packetInterval, record->spaceInterval);
+            printf("Exporter SysID: %u,	Generic Sampler: algorithm: %u, packet interval: %u, packet space: %u\n", sampler_record->exporter_sysid,
+                   sampler_record->algorithm, sampler_record->packetInterval, sampler_record->spaceInterval);
         } else {
-            printf("Exporter SysID: %u,	Sampler: algorithm: %u, packet interval: %u, packet space: %u\n", record->exporter_sysid, record->algorithm,
-                   record->packetInterval, record->spaceInterval);
+            printf("Exporter SysID: %u,	Sampler: algorithm: %u, packet interval: %u, packet space: %u\n", sampler_record->exporter_sysid,
+                   sampler_record->algorithm, sampler_record->packetInterval, sampler_record->spaceInterval);
         }
     }
 #endif
 
     return 1;
-}  // End of AddSamplerInfo
+}  // End of AddSamplerRecord
 
 int AddExporterStat(exporter_stats_record_t *stat_record) {
     if (stat_record->header.size < sizeof(exporter_stats_record_t)) {
@@ -243,7 +263,7 @@ int AddExporterStat(exporter_stats_record_t *stat_record) {
     int use_copy;
     exporter_stats_record_t *rec;
     if (((ptrdiff_t)stat_record & 0x7) != 0) {
-        rec = malloc(stat_record->header.size);
+        rec = (exporter_stats_record_t *)malloc(stat_record->header.size);
         if (!rec) {
             LogError("malloc() error in %s line %d: %s\n", __FILE__, __LINE__, strerror(errno));
             exit(255);
@@ -255,6 +275,7 @@ int AddExporterStat(exporter_stats_record_t *stat_record) {
         use_copy = 0;
     }
 
+    if (exporter_list == NULL) InitExporterList();
     for (int i = 0; i < rec->stat_count; i++) {
         uint32_t id = rec->stat[i].sysid;
         if (id >= MAX_EXPORTERS) {
@@ -288,24 +309,28 @@ exporter_t *GetExporterInfo(int exporterID) {
 
 }  // End of GetExporter
 
-void ExportExporterList(nffile_t *nffile) {
+dataBlock_t *ExportExporterList(nffile_t *nffile, dataBlock_t *dataBlock) {
     // sysid 0 unused -> no exporter available
+
+    if (exporter_list == NULL) return dataBlock;
+
     int i = 1;
     while (i < MAX_EXPORTERS && exporter_list[i] != NULL) {
         exporter_info_record_t *exporter;
         sampler_t *sampler;
 
         exporter = &exporter_list[i]->info;
-        AppendToBuffer(nffile, (void *)exporter, exporter->header.size);
+        dataBlock = AppendToBuffer(nffile, dataBlock, (void *)exporter, exporter->header.size);
 
         sampler = exporter_list[i]->sampler;
         while (sampler) {
-            AppendToBuffer(nffile, (void *)&(sampler->record), sampler->record.size);
+            dataBlock = AppendToBuffer(nffile, dataBlock, (void *)&(sampler->record), sampler->record.size);
             sampler = sampler->next;
         }
 
         i++;
     }
+    return dataBlock;
 
 }  // End of ExportExporterList
 
@@ -317,38 +342,24 @@ void PrintExporters(void) {
         return;
     }
 
+    dataBlock_t *dataBlock = NULL;
     int done = 0;
     int found = 0;
     while (!done) {
         // get next data block from file
-        int ret = ReadBlock(nffile);
-        switch (ret) {
-            case NF_CORRUPT:
-            case NF_ERROR:
-                if (ret == NF_CORRUPT)
-                    LogError("Corrupt data file");
-                else
-                    LogError("Read error: %s", strerror(errno));
-                done = 1;
-                continue;
-                break;
-                // fall through - get next file in chain
-            case NF_EOF:
-                done = 1;
-                continue;
-                break;
-
-                // default:
-                // successfully read block
-        }
-
-        if (nffile->block_header->type != DATA_BLOCK_TYPE_2 && nffile->block_header->type != DATA_BLOCK_TYPE_3) {
-            printf("Skip unknown block type: %u\n", nffile->block_header->type);
+        dataBlock = ReadBlock(nffile, dataBlock);
+        if (dataBlock == NULL) {
+            done = 1;
             continue;
         }
 
-        record_header_t *record = (record_header_t *)nffile->buff_ptr;
-        for (int i = 0; i < nffile->block_header->NumRecords; i++) {
+        if (dataBlock->type != DATA_BLOCK_TYPE_2 && dataBlock->type != DATA_BLOCK_TYPE_3) {
+            printf("Skip unknown block type: %u\n", dataBlock->type);
+            continue;
+        }
+
+        record_header_t *record = GetCursor(dataBlock);
+        for (int i = 0; i < dataBlock->NumRecords; i++) {
             switch (record->type) {
                 case LegacyRecordType1:
                 case LegacyRecordType2:
@@ -357,23 +368,29 @@ void PrintExporters(void) {
                 case ExporterInfoRecordType:
                     found = 1;
                     if (!AddExporterInfo((exporter_info_record_t *)record)) {
-                        LogError("Failed to add Exporter Record\n");
+                        LogError("Failed to add exporter record\n");
                     }
                     break;
                 case ExporterStatRecordType:
                     AddExporterStat((exporter_stats_record_t *)record);
                     break;
                 case SamplerRecordType:
-                    if (!AddSamplerInfo((sampler_record_t *)record)) {
-                        LogError("Failed to add Sampler Record\n");
+                    if (!AddSamplerRecord((sampler_record_t *)record)) {
+                        LogError("Failed to add sampler record\n");
+                    }
+                    break;
+                case SamplerLegacyRecordType:
+                    if (!AddSamplerLegacyRecord((samplerV0_record_t *)record)) {
+                        LogError("Failed to add legacy sampler record\n");
                     }
                     break;
             }
             // Advance pointer by number of bytes for netflow record
-            record = (record_header_t *)((pointer_addr_t)record + record->size);
+            record = (record_header_t *)((void *)record + record->size);
         }
     }
 
+    FreeDataBlock(dataBlock);
     CloseFile(nffile);
     DisposeFile(nffile);
     if (!found) {
@@ -397,23 +414,22 @@ void PrintExporters(void) {
             uint32_t _ip = htonl(exporter->ip.V4);
             inet_ntop(AF_INET, &_ip, ipstr, sizeof(ipstr));
             if (exporter_list[i]->flows)
-                printf("SysID: %u, IP: %16s, version: %u, ID: %2u, Sequence failures: %u, packets: %llu, flows: %llu\n", exporter->sysid, ipstr,
-                       exporter->version, exporter->id, exporter_list[i]->sequence_failure, (long long unsigned)exporter_list[i]->packets,
-                       (long long unsigned)exporter_list[i]->flows);
+                printf("SysID: %u, IP: %16s, version: %s, ID: %2u, Sequence failures: %u, packets: %llu, flows: %llu\n", exporter->sysid, ipstr,
+                       getVersionString(exporter->version), exporter->id, exporter_list[i]->sequence_failure,
+                       (long long unsigned)exporter_list[i]->packets, (long long unsigned)exporter_list[i]->flows);
             else
-                printf("SysID: %u, IP: %16s, version: %u, ID: %2u - no flows sent\n", exporter->sysid, ipstr, exporter->version, exporter->id);
+                printf("SysID: %u, IP: %16s, version: %s, ID: %2u - no flows sent\n", exporter->sysid, ipstr, getVersionString(exporter->version),
+                       exporter->id);
 
         } else if (exporter->sa_family == AF_INET6) {
-            uint64_t _ip[2];
-            _ip[0] = htonll(exporter->ip.V6[0]);
-            _ip[1] = htonll(exporter->ip.V6[1]);
+            uint64_t _ip[2] = {htonll(exporter->ip.V6[0]), htonll(exporter->ip.V6[1])};
             inet_ntop(AF_INET6, &_ip, ipstr, sizeof(ipstr));
             if (exporter_list[i]->flows)
-                printf("SysID: %u, IP: %40s, version: %u, ID: %2u, Sequence failures: %u, packets: %llu, flows: %llu\n ", exporter->sysid, ipstr,
-                       exporter->version, exporter->id, exporter_list[i]->sequence_failure, (long long unsigned)exporter_list[i]->packets,
-                       (long long unsigned)exporter_list[i]->flows);
+                printf("SysID: %u, IP: %40s, version: %s, ID: %2u, Sequence failures: %u, packets: %llu, flows: %llu\n ", exporter->sysid, ipstr,
+                       getVersionString(exporter->version), exporter->id, exporter_list[i]->sequence_failure,
+                       (long long unsigned)exporter_list[i]->packets, (long long unsigned)exporter_list[i]->flows);
             else
-                printf("SysID: %u, IP: %40s, version: %u, ID: %2u\n ", exporter->sysid, ipstr, exporter->version, exporter->id);
+                printf("SysID: %u, IP: %40s, version: %s, ID: %2u\n ", exporter->sysid, ipstr, getVersionString(exporter->version), exporter->id);
         } else {
             strncpy(ipstr, "<unknown>", IP_STRING_LEN);
             printf("**** Exporter IP version unknown ****\n");
