@@ -64,7 +64,7 @@
 #include <unistd.h>
 
 #include "bookkeeper.h"
-#include "collector.h"
+#include "conf/nfconf.h"
 #include "config.h"
 #include "daemon.h"
 #include "expire.h"
@@ -74,7 +74,6 @@
 #include "flowsend.h"
 #include "flowtree.h"
 #include "metric.h"
-#include "nfconf.h"
 #include "nfdump.h"
 #include "nffile.h"
 #include "nfnet.h"
@@ -103,8 +102,8 @@ static int launcher_pid;
 static pthread_mutex_t m_done = PTHREAD_MUTEX_INITIALIZER;
 static pthread_cond_t terminate = PTHREAD_COND_INITIALIZER;
 
-uint32_t linktype;
-uint32_t linkoffset;
+static option_t nfpcapdOption[] = {
+    {.name = "fat", .valBool = 0, .flags = OPTDEFAULT}, {.name = "payload", .valBool = 0, .flags = OPTDEFAULT}, {.name = NULL}};
 
 /*
  * Function prototypes
@@ -117,8 +116,6 @@ static int setup_pcap_file(packetParam_t *param, char *pcap_file, char *filter,
 
 static void WaitDone(void);
 
-static int scanOptions(flowParam_t *flowParam, char *options);
-
 /*
  * Functions
  */
@@ -133,6 +130,7 @@ static void usage(char *name) {
         "-r pcapfile\tread packets from file\n"
         "-b num\tset socket buffer size in MB. (default 20MB)\n"
         "-B num\tset the node cache size. (default 524288)\n"
+        "-d\t\tDe-duplicate packets with window size 8.\n"
         "-s snaplen\tset the snapshot length - default 1522\n"
         "-e active,inactive\tset the active,inactive flow expire time (s) - default 300,60\n"
         "-o options \tAdd flow options, separated with ','. Available: 'fat', 'payload'\n"
@@ -145,6 +143,7 @@ static void usage(char *name) {
         "-I Ident\tset the ident string for stat file. (default 'none')\n"
         "-P pidfile\tset the PID file\n"
         "-t time frame\tset the time window to rotate pcap/nfcapd file\n"
+        "-W workers\toptionally set the number of workers to compress flows\n"
         "-z=lzo\t\tLZO compress flows in output file.\n"
         "-z=bz2\t\tBZIP2 compress flows in output file.\n"
         "-z=lz4[:level]\tLZ4 compress flows in output file.\n"
@@ -191,7 +190,7 @@ static int setup_pcap_file(packetParam_t *param, char *pcap_file, char *filter,
         }
     }
 
-    linktype = pcap_datalink(handle);
+    int linktype = pcap_datalink(handle);
     switch (linktype) {
         case DLT_RAW:
         case DLT_PPP:
@@ -257,30 +256,10 @@ static void WaitDone(void) {
 
 }  // End of WaitDone
 
-static int scanOptions(flowParam_t *flowParam, char *options) {
-    char *option = strtok(options, ",");
-    while (option != NULL) {
-        if (strncasecmp(option, "fat", 4) == 0) {
-            flowParam->extendedFlow = 1;
-            dbg_printf("Found extended flow option\n");
-        } else if (strncasecmp(option, "payload", 8) == 0) {
-            flowParam->addPayload = 1;
-            dbg_printf("Found payload option\n");
-        } else {
-            LogError("Unknown option: %s", option);
-            return -1;
-        }
-        option = strtok(NULL, ",");
-    }
-
-    return 0;
-
-}  // End of scanOption
-
 int main(int argc, char *argv[]) {
     sigset_t signal_set;
     struct sigaction sa;
-    int c, snaplen, bufflen, err, do_daemonize;
+    int c, snaplen, bufflen, err, do_daemonize, doDedup;
     int subdir_index, compress, expire, cache_size, buff_size;
     int activeTimeout, inactiveTimeout, metricInterval, workers;
     dirstat_t *dirstat;
@@ -293,6 +272,7 @@ int main(int argc, char *argv[]) {
     snaplen = 1522;
     bufflen = 0;
     do_daemonize = 0;
+    doDedup = 0;
     launcher_pid = 0;
     device = NULL;
     pcapfile = NULL;
@@ -319,7 +299,7 @@ int main(int argc, char *argv[]) {
     inactiveTimeout = 0;
     workers = 0;
 
-    while ((c = getopt(argc, argv, "b:B:C:De:g:hH:I:i:j:l:m:o:p:P:r:s:S:T:t:u:vVw:yz::")) != EOF) {
+    while ((c = getopt(argc, argv, "b:B:C:dDe:g:hH:I:i:j:l:m:o:p:P:r:s:S:T:t:u:vVw:W:yz::")) != EOF) {
         switch (c) {
             struct stat fstat;
             case 'h':
@@ -341,6 +321,9 @@ int main(int argc, char *argv[]) {
                     configFile = optarg;
                 }
                 break;
+            case 'd':
+                doDedup = 1;
+                break;
             case 'D':
                 do_daemonize = 1;
                 break;
@@ -570,10 +553,13 @@ int main(int argc, char *argv[]) {
     flowParam_t flowParam = {0};
     flushParam.extensionFormat = time_extension;
     flowParam.extensionFormat = time_extension;
+    packetParam.doDedup = doDedup;
 
-    if (options && scanOptions(&flowParam, options) < 0) {
+    if (scanOptions(nfpcapdOption, options) == 0) {
         exit(EXIT_FAILURE);
     }
+    OptGetBool(nfpcapdOption, "fat", &flowParam.extendedFlow);
+    OptGetBool(nfpcapdOption, "payload", &flowParam.addPayload);
 
     if ((datadir && sendHost) || (!datadir && !sendHost)) {
         LogError("Specify either a local directory or a remote host to dump flows.");
@@ -778,8 +764,8 @@ int main(int argc, char *argv[]) {
 
     CloseMetric();
 
-    LogInfo("Total: Processed: %u, skipped: %u, short caplen: %u, unknown: %u\n", packetParam.proc_stat.packets, packetParam.proc_stat.skipped,
-            packetParam.proc_stat.short_snap, packetParam.proc_stat.unknown);
+    LogInfo("Total: Processed: %u, skipped: %u, short caplen: %u, unknown: %u, duplicates: %llu\n", packetParam.proc_stat.packets,
+            packetParam.proc_stat.skipped, packetParam.proc_stat.short_snap, packetParam.proc_stat.unknown, packetParam.proc_stat.duplicates);
 
     if (pidfile) remove_pid(pidfile);
 
