@@ -93,6 +93,15 @@ typedef struct vlan_hdr_s {
     uint16_t type;
 } vlan_hdr_t;
 
+// remember the last SlotSize packets with len and hash
+// for duplicate check
+#define SlotSize 8
+static struct {
+    uint32_t len;
+    uint64_t hash;
+} lastPacketStat[SlotSize] = {0};
+static uint32_t packetSlot = 0;
+
 static time_t lastRun = 0;  // remember last run to idle cache
 
 static inline void SetServer_latency(struct FlowNode *node);
@@ -109,6 +118,22 @@ static inline void ProcessICMPFlow(packetParam_t *packetParam, struct FlowNode *
 
 static inline void ProcessOtherFlow(packetParam_t *packetParam, struct FlowNode *NewNode, void *payload, size_t payloadSize);
 
+#include "metrohash.c"
+
+static int is_duplicate(const uint8_t *data_ptr, const uint32_t len) {
+    uint64_t hash = metrohash64_1(data_ptr, len, 0);
+
+    for (int i = 0; i < SlotSize; i++) {
+        if (lastPacketStat[i].len == len && lastPacketStat[i].hash == hash) return 1;
+    }
+
+    // not found - add to next slot round robin
+    lastPacketStat[packetSlot].len = len;
+    lastPacketStat[packetSlot].hash = hash;
+    packetSlot = (packetSlot + 1) & (SlotSize - 1);
+    return 0;
+}  // End of is_duplicate
+
 pcapfile_t *OpenNewPcapFile(pcap_t *p, char *filename, pcapfile_t *pcapfile) {
     if (!pcapfile) {
         // Create struct
@@ -205,62 +230,54 @@ void RotateFile(pcapfile_t *pcapfile, time_t t_CloseRename, int live) {
 
 }  // End of RotateFile
 
-// Server latency = t(SYN Server) - t(SYN CLient)
+// Server latency = t(SYN ACK Server) - t(SYN CLient)
 static inline void SetServer_latency(struct FlowNode *node) {
-    struct FlowNode *Client_node;
-    uint64_t latency;
-
-    Client_node = node->rev_node;
+    struct FlowNode *Client_node = node->rev_node;
     if (!Client_node) return;
 
-    latency = ((uint64_t)node->t_first.tv_sec * (uint64_t)1000000 + (uint64_t)node->t_first.tv_usec) -
-              ((uint64_t)Client_node->t_first.tv_sec * (uint64_t)1000000 + (uint64_t)Client_node->t_first.tv_usec);
+    uint64_t latency = ((uint64_t)node->t_first.tv_sec * (uint64_t)1000000 + (uint64_t)node->t_first.tv_usec) -
+                       ((uint64_t)Client_node->t_first.tv_sec * (uint64_t)1000000 + (uint64_t)Client_node->t_first.tv_usec);
 
     node->latency.server = latency;
     Client_node->latency.server = latency;
+    // set flag, to calc app latency with nex packet from server
+    node->latency.flag = 2;
     // set flag, to calc client latency with nex packet from client
     Client_node->latency.flag = 1;
     dbg_printf("Server latency: %llu\n", (long long unsigned)latency);
 
 }  // End of SetServerClient_latency
 
-// Client latency = t(ACK CLient) - t(SYN Server)
+// Client latency = t(ACK CLient) - t(SYN ACK Server)
 static inline void SetClient_latency(struct FlowNode *node, struct timeval *t_packet) {
-    struct FlowNode *Server_node;
-    uint64_t latency;
+    struct FlowNode *serverNode = node->rev_node;
+    if (!serverNode) return;
 
-    Server_node = node->rev_node;
-    if (!Server_node) return;
-
-    latency = ((uint64_t)t_packet->tv_sec * (uint64_t)1000000 + (uint64_t)t_packet->tv_usec) -
-              ((uint64_t)Server_node->t_first.tv_sec * (uint64_t)1000000 + (uint64_t)Server_node->t_first.tv_usec);
+    uint64_t latency = ((uint64_t)t_packet->tv_sec * (uint64_t)1000000 + (uint64_t)t_packet->tv_usec) -
+                       ((uint64_t)serverNode->t_last.tv_sec * (uint64_t)1000000 + (uint64_t)serverNode->t_last.tv_usec);
 
     node->latency.client = latency;
-    Server_node->latency.client = latency;
+    serverNode->latency.client = latency;
     // reset flag
     node->latency.flag = 0;
-    // set flag, to calc application latency with nex packet from server
-    Server_node->latency.flag = 2;
-    Server_node->latency.t_request = *t_packet;
     dbg_printf("Client latency: %llu\n", (long long unsigned)latency);
 
 }  // End of SetClient_latency
 
 // Application latency = t(ACK Server) - t(ACK CLient)
 void SetApplication_latency(struct FlowNode *node, struct timeval *t_packet) {
-    struct FlowNode *Client_node;
-    uint64_t latency;
+    struct FlowNode *clientNode = node->rev_node;
+    if (!clientNode) return;
 
-    Client_node = node->rev_node;
-    if (!Client_node) return;
-
-    latency = ((uint64_t)t_packet->tv_sec * (uint64_t)1000000 + (uint64_t)t_packet->tv_usec) -
-              ((uint64_t)node->latency.t_request.tv_sec * (uint64_t)1000000 + (uint64_t)node->latency.t_request.tv_usec);
+    uint64_t latency = ((uint64_t)t_packet->tv_sec * (uint64_t)1000000 + (uint64_t)t_packet->tv_usec) -
+                       ((uint64_t)clientNode->t_last.tv_sec * (uint64_t)1000000 + (uint64_t)clientNode->t_last.tv_usec);
 
     node->latency.application = latency;
-    Client_node->latency.application = latency;
+    clientNode->latency.application = latency;
     // reset flag
     node->latency.flag = 0;
+    // set flag, to calc application latency with nex packet from server
+    clientNode->latency.flag = 0;
     dbg_printf("Application latency: %llu\n", (long long unsigned)latency);
 
 }  // End of SetApplication_latency
@@ -311,8 +328,9 @@ static inline struct FlowNode *ProcessIPfrag(packetParam_t *packetParam, const s
         FindNode.flowKey.dst_port = 0;
 
         Node = Lookup_Node(&FindNode);
-        if (!Node) {
+        if (!Node || Node->nodeType != FRAG_NODE) {
             dbg_printf("IP fragment: initial node missing! Skip!\n");
+            packetParam->proc_stat.skipped++;
             return NULL;
         }
 
@@ -373,8 +391,17 @@ static inline void ProcessTCPFlow(packetParam_t *packetParam, struct FlowNode *N
             // flush node to flow thread
             Remove_Node(NewNode);
             Push_Node(packetParam->NodeList, NewNode);
+            return;
         }
 
+#ifdef DEVEL
+        if ((NewNode->flags & 0x3F) == 0x2) {
+            printf("SYN Node\n");
+        }
+        if ((NewNode->flags & 0x37) == 0x12) {
+            printf("SYN ACK Node\n");
+        }
+#endif
         if (packetParam->extendedFlow && Link_RevNode(NewNode)) {
             // if we could link this new node, it is the server answer
             // -> calculate server latency
@@ -385,10 +412,12 @@ static inline void ProcessTCPFlow(packetParam_t *packetParam, struct FlowNode *N
 
     assert(Node->memflag == NODE_IN_USE);
 
-    // check for first client ACK for client latency
+    // check for latency flag
     if (Node->latency.flag == 1) {
         SetClient_latency(Node, &(NewNode->t_first));
     } else if (Node->latency.flag == 2) {
+        dbg_printf("Set App lat slot: %u -> %u diff: %u\n", Node->latency.tsVal, NewNode->latency.tsVal,
+                   NewNode->latency.tsVal - Node->latency.tsVal);
         SetApplication_latency(Node, &(NewNode->t_first));
     }
     // update existing flow
@@ -396,7 +425,19 @@ static inline void ProcessTCPFlow(packetParam_t *packetParam, struct FlowNode *N
     Node->packets++;
     Node->bytes += NewNode->bytes;
     Node->t_last = NewNode->t_last;
+
+    // DEVEL RTT - disabled for now
+#if 0
+    if (NewNode->signal != SIGNAL_FIN && Node->latency.ack && ((NewNode->latency.ack - Node->latency.ack) > 0)) {
+        uint32_t rtt = NewNode->latency.tsVal - Node->latency.tsVal;
+        printf("Node old RTT: %u ", Node->latency.rtt);
+        if (rtt) Node->latency.rtt = Node->latency.rtt ? (Node->latency.rtt + rtt) >> 1 : rtt;
+        dbg_printf("Node rtt: %u, new RTT: %u\n", rtt, Node->latency.rtt);
+    }
+    Node->latency.tsVal = NewNode->latency.tsVal;
+    Node->latency.ack = NewNode->latency.ack;
     dbg_printf("Existing TCP flow: Packets: %u, Bytes: %u\n", Node->packets, Node->bytes);
+#endif
 
     if (Node->payloadSize == 0 && payloadSize > 0 && packetParam->addPayload) {
         dbg_printf("Existing TCP flow: Set payload of size: %zu\n", payloadSize);
@@ -499,7 +540,7 @@ static inline void ProcessOtherFlow(packetParam_t *packetParam, struct FlowNode
 
 }  // End of ProcessOtherFlow
 
-void ProcessPacket(packetParam_t *packetParam, const struct pcap_pkthdr *hdr, const u_char *data) {
+int ProcessPacket(packetParam_t *packetParam, const struct pcap_pkthdr *hdr, const u_char *data) {
     struct FlowNode *Node = NULL;
     uint16_t version, IPproto;
     char s1[64];
@@ -526,6 +567,7 @@ void ProcessPacket(packetParam_t *packetParam, const struct pcap_pkthdr *hdr, co
     // link layer processing
     uint16_t protocol = 0;
     uint32_t linktype = packetParam->linktype;
+    int redoLink = 0;
 REDO_LINK:
     switch (linktype) {
         case DLT_EN10MB:
@@ -535,7 +577,7 @@ REDO_LINK:
             int IEEE802 = protocol <= 1500;
             if (IEEE802) {
                 packetParam->proc_stat.skipped++;
-                return;
+                return 1;
             }
             // unwrap link layer
             dataptr += 14;
@@ -582,7 +624,7 @@ REDO_LINK:
                     break;
                 default:
                     LogInfo("Packet: %u: unsupported DLT_NULL protocol: 0x%x, packet: %u", pkg_cnt, header);
-                    return;
+                    return 1;
             }
         } break;
         case DLT_LINUX_SLL:
@@ -601,12 +643,12 @@ REDO_LINK:
             nflog_hdr_t *nflog_hdr = (nflog_hdr_t *)dataptr;
             if (hdr->caplen < sizeof(nflog_hdr_t)) {
                 LogInfo("Packet: %u: NFLOG: not enough data", pkg_cnt);
-                return;
+                return 1;
             }
 
             if (nflog_hdr->nflog_version != 0) {
                 LogInfo("Packet: %u: unsupported NFLOG version: %d", pkg_cnt, nflog_hdr->nflog_version);
-                return;
+                return 1;
             }
             dbg_printf("Linktype: DLT_NFLOG\n");
             dbg_printf("NFLOG: %s, rid: %u\n", nflog_hdr->nflog_family == 2 ? "IPv4" : "IPv6", ntohs(nflog_hdr->nflog_rid));
@@ -620,7 +662,7 @@ REDO_LINK:
                 if (size % 4 != 0) size += 4 - size % 4;
                 if (size < sizeof(nflog_tlv_t)) {
                     LogInfo("Packet: %u: NFLOG: tlv size error: %u", pkg_cnt, size);
-                    return;
+                    return 1;
                 }
 
                 if (tlv->tlv_type == NFULA_PAYLOAD) {
@@ -636,7 +678,7 @@ REDO_LINK:
             pflog_hdr_t *pfloghdr = (pflog_hdr_t *)dataptr;
             if (hdr->caplen < PFLOG_HDRLEN) {
                 LogInfo("Packet: %u: PFLOG: not enough data", pkg_cnt);
-                return;
+                return 1;
             }
             pflog = malloc(sizeof(pflog_hdr_t));
             memcpy(pflog, pfloghdr, sizeof(pflog_hdr_t));
@@ -647,20 +689,20 @@ REDO_LINK:
         } break;
         default:
             LogInfo("Packet: %u: unsupported link type: 0x%x, packet: %u", pkg_cnt, linktype);
-            return;
+            return 1;
     }
 
 REDO_LINK_PROTO:
     if (dataptr >= eodata) {
         packetParam->proc_stat.short_snap++;
         dbg_printf("Short packet: %u, Check line: %u", hdr->caplen, __LINE__);
-        return;
+        return 1;
     }
     dbg_printf("Next protocol: 0x%x\n", protocol);
     int IEEE802 = protocol <= 1500;
     if (IEEE802) {
         packetParam->proc_stat.skipped++;
-        return;
+        return 1;
     }
     switch (protocol) {
         case ETHERTYPE_IP:    // IPv4
@@ -784,12 +826,29 @@ REDO_IPPROTO:
         size_t size_ip = sizeof(struct ip6_hdr);
 
         dataptr += size_ip;
-        if (dataptr > eodata) {
+        if (dataptr >= eodata) {
             dbg_printf("Short packet: %u, Check line: %u", hdr->caplen, __LINE__);
             packetParam->proc_stat.short_snap++;
             goto END_FUNC;
         }
 
+        // IPv6 duplicate check
+        // dublicate check starts from the IP header over the rest of the packet
+        // vlan, mpls and layer 1 headers are ignored
+        if (unlikely(packetParam->doDedup && redoLink == 0)) {
+            // check for de-dup
+            uint32_t hopLimit = ip6->ip6_ctlun.ip6_un1.ip6_un1_hlim;
+            ip6->ip6_ctlun.ip6_un1.ip6_un1_hlim = 0;
+            uint16_t len = ntohs(ip6->ip6_ctlun.ip6_un1.ip6_un1_plen);
+            if (is_duplicate((const uint8_t *)ip, len + 40)) {
+                packetParam->proc_stat.duplicates++;
+                return 0;
+            }
+            ip6->ip6_ctlun.ip6_un1.ip6_un1_hlim = hopLimit;
+            // prevent recursive dedub checks with IP in IP packets
+            redoLink++;
+        }
+
         // ipv6 Extension headers not processed
         IPproto = ip6->ip6_ctlun.ip6_un1.ip6_un1_nxt;
         dbg_printf("Packet IPv6, SRC %s, DST %s\n", inet_ntop(AF_INET6, &ip6->ip6_src, s1, sizeof(s1)),
@@ -798,10 +857,12 @@ REDO_IPPROTO:
         if (!Node) Node = New_Node();
         Node->flowKey.version = AF_INET6;
         Node->t_first.tv_sec = hdr->ts.tv_sec;
-        Node->t_first.tv_usec = hdr->ts.tv_usec;
         Node->t_last.tv_sec = hdr->ts.tv_sec;
+        Node->t_first.tv_usec = hdr->ts.tv_usec;
         Node->t_last.tv_usec = hdr->ts.tv_usec;
         Node->bytes = ntohs(ip6->ip6_plen) + size_ip;
+        Node->ttl = ip6->ip6_ctlun.ip6_un1.ip6_un1_hlim;
+        Node->fragmentFlags = 0;
 
         // keep compiler happy - gets optimized out anyway
         void *p = (void *)&ip6->ip6_src;
@@ -826,6 +887,25 @@ REDO_IPPROTO:
             goto END_FUNC;
         }
 
+        // IPv4 duplicate check
+        // dublicate check starts from the IP header over the rest of the packet
+        // vlan, mpls and layer 1 headers are ignored
+        if (unlikely(packetParam->doDedup && redoLink == 0)) {
+            // check for de-dup
+            uint32_t ttl = ip->ip_ttl;
+            uint32_t sum = ip->ip_sum;
+            ip->ip_ttl = 0;
+            ip->ip_sum = 0;
+            if (is_duplicate((const uint8_t *)ip, ntohs(ip->ip_len))) {
+                packetParam->proc_stat.duplicates++;
+                return 0;
+            }
+            ip->ip_ttl = ttl;
+            ip->ip_sum = sum;
+            // prevent recursive dedub checks with IP in IP packets
+            redoLink++;
+        }
+
         IPproto = ip->ip_p;
         dbg_printf("Packet IPv4 SRC %s, DST %s\n", inet_ntop(AF_INET, &ip->ip_src, s1, sizeof(s1)), inet_ntop(AF_INET, &ip->ip_dst, s2, sizeof(s2)));
 
@@ -845,20 +925,21 @@ REDO_IPPROTO:
             eodata = dataptr + Node->payloadSize;
             Node->payload = NULL;
             Node->payloadSize = 0;
-            Node->fragmentFlags |= IP_MF;
+            Node->fragmentFlags |= flagMF;
         } else {
             if (!Node) Node = New_Node();
             Node->flowKey.version = AF_INET;
             Node->t_first.tv_sec = hdr->ts.tv_sec;
-            Node->t_first.tv_usec = hdr->ts.tv_usec;
             Node->t_last.tv_sec = hdr->ts.tv_sec;
+            Node->t_first.tv_usec = hdr->ts.tv_usec;
             Node->t_last.tv_usec = hdr->ts.tv_usec;
             Node->bytes = ntohs(ip->ip_len);
-            Node->fragmentFlags |= (ip_off & IP_DF);
+            if (ip_off & IP_DF) Node->fragmentFlags |= flagDF;
 
             Node->flowKey.src_addr.v4 = ntohl(ip->ip_src.s_addr);
             Node->flowKey.dst_addr.v4 = ntohl(ip->ip_dst.s_addr);
         }
+        Node->ttl = ip->ip_ttl;
     } else {
         dbg_printf("ProcessPacket() Unsupported protocol version: %i\n", version);
         packetParam->proc_stat.unknown++;
@@ -945,7 +1026,49 @@ REDO_IPPROTO:
             if (tcp->th_flags & TH_RST) printf("RST ");
             printf("\n");
 #endif
+            Node->signal = tcp->th_flags & TH_FIN || tcp->th_flags & TH_RST;
 
+            // RTT DEVEL - disabled for now
+
+#if 0
+            if (tcp->th_off > 5) {
+                uint32_t optLen = (tcp->th_off - 5) << 2;
+                uint8_t *optData = dataptr - (ptrdiff_t)optLen;
+                dbg_printf("TCP HLen: %u, tcp option len: %u\n", tcp->th_off, optLen);
+                while (optData < dataptr) {
+                    uint8_t opt = *optData++;
+                    uint8_t optLen = 0;
+                    switch (opt) {
+                        case 0:  // End of options
+                        case 1:  // NOP
+                            continue;
+                            break;
+                        case 8:  // TS
+                            optLen = *optData++;
+                            if (optLen != 10) {
+                                // skip unknown TS len
+                                LogError("TCP option TS len error: %u", optLen);
+                            } else {
+                                uint32_t tsVal = ntohl(*(uint32_t *)optData);
+                                Node->latency.tsVal = tsVal;
+                                optData += 4;
+                                uint32_t tsEcr = ntohl(*(uint32_t *)optData);
+                                optData += 4;
+                                dbg_printf("TS tcp option: %u, optLen: %u, tsVal: %u, tsEcr: %u\n", opt, optLen, tsVal, tsEcr);
+                                Node->latency.ack = ntohl(tcp->th_ack);
+                                continue;
+                            }
+                            break;
+                        default:
+                            optLen = *optData++;
+                            optData += (optLen - 2);
+                            dbg_printf("Next tcp option: %u, optLen: %u, size left: %lu\n", opt, optLen, (dataptr - optData));
+                    }
+                }
+            } else {
+                dbg_printf("OptLen: %u, no tcp option decoded\n", tcp->th_off);
+            }
+#endif
             Node->flags = tcp->th_flags;
             Node->flowKey.src_port = ntohs(tcp->th_sport);
             Node->flowKey.dst_port = ntohs(tcp->th_dport);
@@ -1137,4 +1260,5 @@ END_FUNC:
         lastRun = hdr->ts.tv_sec;
     }
 
+    return 1;
 }  // End of ProcessPacket
