@@ -54,12 +54,12 @@
 #include <stdio_ext.h>
 #endif
 
+#include "filter/filter.h"
 #include "flist.h"
 #include "nbar.h"
 #include "nfd_raw.h"
 #include "nfdump.h"
 #include "nffile.h"
-#include "nftree.h"
 #include "nfxV3.h"
 #include "send_net.h"
 #include "send_v5.h"
@@ -74,13 +74,14 @@
 #ifdef HAVE___FPURGE
 #define FPURGE __fpurge
 #endif
+#ifndef FPURGE
 #ifdef HAVE_FPURGE
 #define FPURGE fpurge
 #endif
+#endif
 
 /* Local Variables */
 static int verbose = 0;
-static FilterEngine_t *Engine;
 
 static send_peer_t peer;
 static uint32_t recordCnt = 0;
@@ -89,7 +90,7 @@ static uint32_t sequence = 0;
 /* Function Prototypes */
 static void usage(char *name);
 
-static void send_data(timeWindow_t *timeWindow, uint32_t count, unsigned int delay, int confirm, int netflow_version, int distribution);
+static void send_data(void *engine, timeWindow_t *timeWindow, uint64_t count, unsigned int delay, int confirm, int netflow_version, int distribution);
 
 static int FlushBuffer(int confirm);
 
@@ -198,7 +199,8 @@ static int FlushBuffer(int confirm) {
     return sendto(peer.sockfd, peer.send_buffer, len, 0, (struct sockaddr *)&(peer.dstaddr), peer.addrlen);
 }  // End of FlushBuffer
 
-static void send_data(timeWindow_t *timeWindow, uint32_t limitRecords, unsigned int delay, int confirm, int netflow_version, int distribution) {
+static void send_data(void *engine, timeWindow_t *timeWindow, uint64_t limitRecords, unsigned int delay, int confirm, int netflow_version,
+                      int distribution) {
     nffile_t *nffile;
     uint64_t twin_msecFirst, twin_msecLast;
 
@@ -223,10 +225,11 @@ static void send_data(timeWindow_t *timeWindow, uint32_t limitRecords, unsigned
         LogError("GetNextFile() error in %s line %d: %s\n", __FILE__, __LINE__, strerror(errno));
         return;
     }
-    if (nffile == EMPTY_LIST) {
+    if (nffile == NULL) {
         LogError("Empty file list. No files to process\n");
         return;
     }
+    FilterSetParam(engine, nffile->ident, NOGEODB);
 
     peer.send_buffer = malloc(UDP_PACKET_SIZE);
     peer.flush = 0;
@@ -249,54 +252,40 @@ static void send_data(timeWindow_t *timeWindow, uint32_t limitRecords, unsigned
             break;
     }
 
-    uint32_t numflows = 0;
+    recordHandle_t *recordHandle = calloc(1, sizeof(recordHandle_t));
+    if (!recordHandle) {
+        LogError("malloc() error in %s line %d: %s\n", __FILE__, __LINE__, strerror(errno));
+        return;
+    }
+
+    dataBlock_t *dataBlock = NULL;
+    uint64_t numflows = 0;
+    uint64_t processed = 0;
     int done = 0;
-
-    // setup Filter Engine to point to master_record, as any record read from file
-    // is expanded into this record
-    master_record_t *master_record = malloc(sizeof(master_record_t));
-    Engine->nfrecord = (uint64_t *)master_record;
-    Engine->ident = nffile->ident;
-
     while (!done) {
         // get next data block from file
-        int ret = ReadBlock(nffile);
-
-        switch (ret) {
-            case NF_CORRUPT:
-            case NF_ERROR:
-                if (ret == NF_CORRUPT)
-                    LogError("Skip corrupt data file '%s'\n", nffile->fileName);
-                else
-                    LogError("Read error in file '%s': %s\n", nffile->fileName, strerror(errno));
-                // fall through - get next file in chain
-            case NF_EOF: {
-                nffile_t *next = GetNextFile(nffile);
-                if (next == EMPTY_LIST) {
-                    done = 1;
-                }
-                if (next == NULL) {
-                    done = 1;
-                    LogError("Unexpected end of file list\n");
-                }
-                // else continue with next file
-                Engine->ident = nffile->ident;
-                continue;
-
-            } break;  // not really needed
+        dataBlock = ReadBlock(nffile, dataBlock);
+        if (dataBlock == NULL) {
+            nffile_t *next = GetNextFile(nffile);
+            if (next == NULL) {
+                done = 1;
+            }
+            FilterSetParam(engine, nffile->ident, NOGEODB);
+            // else continue with next file
+            continue;
         }
 
-        if (nffile->block_header->type != DATA_BLOCK_TYPE_2 && nffile->block_header->type != DATA_BLOCK_TYPE_3) {
-            LogError("Can't process block type %u. Skip block.\n", nffile->block_header->type);
+        if (dataBlock->type != DATA_BLOCK_TYPE_2 && dataBlock->type != DATA_BLOCK_TYPE_3) {
+            LogError("Can't process block type %u. Skip block.\n", dataBlock->type);
             continue;
         }
 
         // cnt is the number of blocks, which matched the filter
         // and added to the output buffer
-        record_header_t *record_ptr = nffile->buff_ptr;
+        record_header_t *record_ptr = GetCursor(dataBlock);
         uint32_t sumSize = 0;
-        for (int i = 0; i < nffile->block_header->NumRecords; i++) {
-            if ((sumSize + record_ptr->size) > ret || (record_ptr->size < sizeof(record_header_t))) {
+        for (int i = 0; i < dataBlock->NumRecords; i++) {
+            if ((sumSize + record_ptr->size) > dataBlock->size || (record_ptr->size < sizeof(record_header_t))) {
                 LogError("Corrupt data file. Inconsistent block size in %s line %d\n", __FILE__, __LINE__);
                 exit(255);
             }
@@ -305,15 +294,23 @@ static void send_data(timeWindow_t *timeWindow, uint32_t limitRecords, unsigned
             switch (record_ptr->type) {
                 case V3Record: {
                     int match;
-                    memset((void *)master_record, 0, sizeof(master_record_t));
-                    ExpandRecord_v3((recordHeaderV3_t *)record_ptr, master_record);
+                    processed++;
+                    MapRecordHandle(recordHandle, (recordHeaderV3_t *)record_ptr, processed);
+
                     // Time based filter
                     // if no time filter is given, the result is always true
-                    match = twin_msecFirst && (master_record->msecFirst < twin_msecFirst || master_record->msecLast > twin_msecLast) ? 0 : 1;
+                    if (twin_msecFirst) {
+                        EXgenericFlow_t *genericFlow = (EXgenericFlow_t *)recordHandle->extensionList[EXgenericFlowID];
+                        match = (genericFlow->msecFirst < twin_msecFirst || genericFlow->msecLast > twin_msecLast) ? 0 : 1;
+                    } else {
+                        match = 1;
+                    }
+
+                    // limit recordcount
                     match &= limitRecords ? numflows < limitRecords : 1;
 
                     // filter netflow record with user supplied filter
-                    if (match) match = (*Engine->FilterEngine)(Engine);
+                    if (match) match = FilterRecord(engine, recordHandle);
 
                     if (match == 0) {  // record failed to pass all filters
                         // go to next record
@@ -324,10 +321,10 @@ static void send_data(timeWindow_t *timeWindow, uint32_t limitRecords, unsigned
                     int again = 0;
                     switch (netflow_version) {
                         case 5:
-                            again = Add_v5_output_record(master_record, &peer);
+                            again = Add_v5_output_record(recordHandle, &peer);
                             break;
                         case 9:
-                            again = Add_v9_output_record(master_record, &peer);
+                            again = Add_v9_output_record(recordHandle, &peer);
                             break;
                         case 250:
                             again = Add_nfd_output_record(record_ptr, &peer);
@@ -355,10 +352,10 @@ static void send_data(timeWindow_t *timeWindow, uint32_t limitRecords, unsigned
                     if (again) {
                         switch (netflow_version) {
                             case 5:
-                                again = Add_v5_output_record(master_record, &peer);
+                                again = Add_v5_output_record(recordHandle, &peer);
                                 break;
                             case 9:
-                                again = Add_v9_output_record(master_record, &peer);
+                                again = Add_v9_output_record(recordHandle, &peer);
                                 break;
                             case 250:
                                 again = Add_nfd_output_record(record_ptr, &peer);
@@ -382,8 +379,10 @@ static void send_data(timeWindow_t *timeWindow, uint32_t limitRecords, unsigned
 
             // z-parameter
             // first and last are line (tstart and tend) timestamp with milliseconds
-            // first = (double)master_record->msecFirst / 1000.0;
-            double last = (double)master_record->msecLast / 1000.0;
+            // first = (double)genericFlow->msecFirst / 1000.0;
+            double last = 0.0;
+            EXgenericFlow_t *genericFlow = (EXgenericFlow_t *)recordHandle->extensionList[EXgenericFlowID];
+            if (genericFlow) last = (double)genericFlow->msecLast / 1000.0;
 
             gettimeofday(&currentTime, NULL);
             double now = (double)currentTime.tv_sec + (double)currentTime.tv_usec / 1000000;
@@ -441,7 +440,7 @@ int main(int argc, char **argv) {
     struct stat stat_buff;
     char *ffile, *filter, *tstring;
     int c, confirm, ffd, ret, netflow_version, distribution;
-    unsigned int delay, count, sockbuff_size;
+    unsigned int delay, sockbuff_size;
     timeWindow_t *timeWindow;
     flist_t flist;
 
@@ -457,12 +456,12 @@ int main(int argc, char **argv) {
     peer.sockfd = 0;
 
     delay = 1;
-    count = 0;
     sockbuff_size = 0;
     netflow_version = 9;
     verbose = 0;
     confirm = 0;
     distribution = 0;
+    uint64_t count = 0;
     while ((c = getopt(argc, argv, "46EhH:i:K:L:p:S:d:c:b:j:r:f:t:v:z:VY")) != EOF) {
         switch (c) {
             case 'h':
@@ -592,8 +591,8 @@ int main(int argc, char **argv) {
 
     if (!filter) filter = "any";
 
-    Engine = CompileFilter(filter);
-    if (!Engine) exit(254);
+    void *engine = CompileFilter(filter);
+    if (!engine) exit(254);
 
     if (peer.mcast)
         peer.sockfd =
@@ -613,7 +612,7 @@ int main(int argc, char **argv) {
     queue_t *fileList = SetupInputFileSequence(&flist);
     if (!Init_nffile(1, fileList)) exit(254);
 
-    send_data(timeWindow, count, delay, confirm, netflow_version, distribution);
+    send_data(engine, timeWindow, count, delay, confirm, netflow_version, distribution);
 
     return 0;
 }
