@@ -1,5 +1,5 @@
 /*
- *  Copyright (c) 2009-2023, Peter Haag
+ *  Copyright (c) 2009-2024, Peter Haag
  *  Copyright (c) 2004-2008, SWITCH - Teleinformatikdienste fuer Lehre und Forschung
  *  All rights reserved.
  *
@@ -46,10 +46,10 @@
 #include <unistd.h>
 
 #include "config.h"
+#include "filter/filter.h"
 #include "flist.h"
 #include "nfdump.h"
 #include "nffile.h"
-#include "nftree.h"
 #include "rbtree.h"
 #include "util.h"
 
@@ -59,6 +59,12 @@ extern char influxdb_url[1024];
 static char influxdb_measurement[] = "nfsen_stats";
 #endif
 
+#if HAVE_RRDVERSION > 8
+#define rrdchar const char
+#else
+#define rrdchar char
+#endif
+
 /* imported vars */
 extern char yyerror_buff[256];
 
@@ -205,14 +211,14 @@ static void SetupProfileChannels(char *profile_datadir, char *profile_statdir, p
     if (verify_only)
         printf("Check filter for channel %s in profile '%s' in group '%s': ", profile_param->channelname, profile_param->profilename,
                profile_param->profilegroup);
-    FilterEngine_t *engine = CompileFilter(filter);
+    void *engine = CompileFilter(filter);
 
     if (!engine) {
         printf("\n");
         LogError("*** Compiling filter failed for channel %s in profile '%s' in group '%s'.", profile_param->channelname, profile_param->profilename,
                  profile_param->profilegroup);
         LogError("*** File: %s", path);
-        LogError("*** Error: %s\n", yyerror_buff);
+        // XXX LogError("*** Error: %s\n", yyerror_buff);
         LogError("*** Failed Filter: %s", filter);
         free(filter);
         return;
@@ -221,6 +227,7 @@ static void SetupProfileChannels(char *profile_datadir, char *profile_statdir, p
 
     if (verify_only) {
         printf("ok.\n");
+        DisposeFilter(engine);
         return;
     }
 
@@ -240,6 +247,7 @@ static void SetupProfileChannels(char *profile_datadir, char *profile_statdir, p
     char *ofile = NULL;
     char *wfile = NULL;
     nffile_t *nffile = NULL;
+    dataBlock_t *dataBlock = NULL;
     if ((profile_param->profiletype & 4) == 0) {  // no shadow profile
         int is_alert = (profile_param->profiletype & 8) == 8;
         if (!is_alert && subdir_index && strlen(filename) == 19 && (strncmp(filename, "nfcapd.", 7) == 0)) {
@@ -286,6 +294,7 @@ static void SetupProfileChannels(char *profile_datadir, char *profile_statdir, p
             return;
         }
         SetIdent(nffile, profile_param->channelname);
+        dataBlock = WriteBlock(nffile, NULL);
     }
 
     snprintf(path, MAXPATHLEN - 1, "%s/%s/%s/%s.rrd", profile_statdir, profile_param->profilegroup, profile_param->profilename,
@@ -315,6 +324,7 @@ static void SetupProfileChannels(char *profile_datadir, char *profile_statdir, p
     profile_channels[num_channels].dirstat_path = strdup(path);
     profile_channels[num_channels].type = profile_param->profiletype;
     profile_channels[num_channels].nffile = nffile;
+    profile_channels[num_channels].dataBlock = dataBlock;
 
     memset((void *)&profile_channels[num_channels].stat_record, 0, sizeof(stat_record_t));
 
@@ -445,7 +455,7 @@ void UpdateRRD(time_t tslot, profile_channel_info_t *channel) {
     opterr = 0;
     rrd_clear_error();
     int i = 0;
-    if ((i = rrd_update(argc, rrd_arg)) != 0) {
+    if ((i = rrd_update(argc, (rrdchar **)rrd_arg)) != 0) {
         LogError("RRD: %s Insert Error: %d %s\n", channel->rrdfile, i, rrd_get_error());
     }
 
