@@ -106,6 +106,13 @@ struct myicmphdr {
                   /* ignore the rest */
 };
 
+/* and GRE (RFC 2890) */
+struct mygreheader { /* only relevant fields */
+    uint8_t flags; /* presence indicators for checksum, key, and sequence number */
+    uint8_t version; /* GRE version */
+    uint16_t protocol_type; /* EtherType code for encapsulated protocol */
+};
+
 typedef struct _SFSample {
     /* exception handler context */
     jmp_buf env;
@@ -149,6 +156,7 @@ typedef struct _SFSample {
     uint8_t *header;
     uint32_t headerLen;
     uint32_t stripped;
+    uint32_t *headerDescriptionStart;
 
     /* header decode */
     int gotIPV4;
@@ -257,7 +265,7 @@ typedef struct _SFSample {
     SFLAddress mpls_nextHop;
 
     /* nat */
-    SFLAddress nat_src;  // EXnselXlateIPv4ID
+    SFLAddress nat_src;  // EXnatXlateIPv4ID
     SFLAddress nat_dst;
     uint32_t nat_src_port;
     uint32_t nat_dst_port;
@@ -271,6 +279,8 @@ typedef struct _SFSample {
 #define SF_ABORT_DECODE_ERROR 2
 #define SF_ABORT_LENGTH_ERROR 3
 
+    int parse_gre;
+
 } SFSample;
 
 #define sampleDataOffset offsetof(SFSample, sampleType)
